@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -15,11 +15,12 @@
  * technology into proprietary software, we sell alternative licenses      *
  * (contact sales@insecure.com).  Dozens of software vendors already       *
  * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * detection, version detection, and the Nmap Scripting Engine.            *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
  * "derivative work" for the purpose of this license if it does any of the *
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
@@ -27,40 +28,32 @@
  *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                * 
+ *   not derivative works.)                                                *
  * o Integrates/includes/aggregates Nmap into a proprietary executable     *
  *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is just meant to        *
- * clarify our interpretation of derived works with some common examples.  *
- * These restrictions only apply when you actually redistribute Nmap.  For *
- * example, nothing stops you from writing and selling a proprietary       *
- * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://nmap.org to download Nmap.                                       *
- *                                                                         *
- * We don't consider these to be added restrictions on top of the GPL, but *
- * just a clarification of how we interpret "derived works" as it applies  *
- * to our GPL-licensed Nmap product.  This is similar to the way Linus     *
- * Torvalds has announced his interpretation of how "derived works"        *
- * applies to Linux kernel modules.  Our interpretation refers only to     *
- * Nmap - we don't speak for any other GPL products.                       *
+ * works of Nmap, as well as other software we distribute under this       *
+ * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+ * but is meant to clarify our interpretation of derived works with some   *
+ * common examples.  Our interpretation applies only to Nmap--we don't     *
+ * speak for other people's GPL works.                                     *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included COPYING.OpenSSL file, and distribute linked      *
- * combinations including the two. You must obey the GNU GPL in all        *
+ * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+ * linked combinations including the two. You must obey the GNU GPL in all *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
  * but you are not obligated to do so.                                     *
@@ -76,17 +69,18 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to fyodor@insecure.org for possible incorporation into the main         *
+ * to nmap-dev@insecure.org for possible incorporation into the main       *
  * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
- * to reuse, modify, and relicense the code.  Nmap will always be          *
- * available Open Source, but this is important because the inability to   *
- * relicense code has caused devastating problems for other Free Software  *
- * projects (such as KDE and NASM).  We also occasionally relicense the    *
- * code to third parties as discussed above.  If you wish to specify       *
- * special license conditions of your contributions, just say so when you  *
- * send them.                                                              *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -97,7 +91,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: NmapOps.cc 9347 2008-08-05 19:28:51Z david $ */
+/* $Id: NmapOps.cc 29958 2012-10-06 20:48:32Z david $ */
 #include "nmap.h"
 #include "nbase.h"
 #include "NmapOps.h"
@@ -116,8 +110,53 @@ NmapOps::NmapOps() {
 }
 
 NmapOps::~NmapOps() {
-  if (datadir) free(datadir);
-  if (xsl_stylesheet) free(xsl_stylesheet);
+  if (xsl_stylesheet) {
+    free(xsl_stylesheet);
+    xsl_stylesheet = NULL;
+  }
+
+  if (reference_FPs) {
+    delete reference_FPs;
+    reference_FPs = NULL;
+  }
+
+  if (dns_servers) {
+    free(dns_servers);
+    dns_servers = NULL;
+  }
+  if (extra_payload) {
+    free(extra_payload);
+    extra_payload = NULL;
+  }
+  if (ipoptions) {
+    free(ipoptions);
+    ipoptions = NULL;
+  }
+  if (portlist) {
+    free(portlist);
+    portlist = NULL;
+  }
+  if (exclude_spec) {
+    free(exclude_spec);
+    exclude_spec = NULL;
+  }
+  if (idleProxy) {
+    free(idleProxy);
+    idleProxy = NULL;
+  }
+  if (datadir) {
+    free(datadir);
+    datadir = NULL;
+  }
+
+#ifndef NOLUA
+  if (scriptversion || script)
+    close_nse();
+  if (scriptargs) {
+    free(scriptargs);
+    scriptargs = NULL;
+  }
+#endif
 }
 
 void NmapOps::ReInit() {
@@ -140,6 +179,14 @@ int NmapOps::SourceSockAddr(struct sockaddr_storage *ss, size_t *ss_len) {
   return 0;
 }
 
+/* Returns a const pointer to the source address if set, or NULL if unset. */
+const struct sockaddr_storage *NmapOps::SourceSockAddr() const {
+  if (sourcesock.ss_family == AF_UNSPEC)
+    return NULL;
+  else
+    return &sourcesock;
+}
+
 /* Note that it is OK to pass in a sockaddr_in or sockaddr_in6 casted
      to sockaddr_storage */
 void NmapOps::setSourceSockAddr(struct sockaddr_storage *ss, size_t ss_len) {
@@ -148,36 +195,46 @@ void NmapOps::setSourceSockAddr(struct sockaddr_storage *ss, size_t ss_len) {
   sourcesocklen = ss_len;
 }
 
-struct in_addr NmapOps::v4source() {
- const struct in_addr *addy = v4sourceip();
-  struct in_addr in;
-  if (addy) return *addy;
-  in.s_addr = 0;
-  return in;
-}
-
-const struct in_addr *NmapOps::v4sourceip() {
-   struct sockaddr_in *sin = (struct sockaddr_in *) &sourcesock;
-  if (sin->sin_family == AF_INET) {
-    return &(sin->sin_addr);
-  }
-  return NULL;
-}
-
-// Number of milliseconds since getStartTime().  The current time is an
+// Number of seconds since getStartTime().  The current time is an
 // optional argument to avoid an extra gettimeofday() call.
-int NmapOps::TimeSinceStartMS(struct timeval *now) {
+float NmapOps::TimeSinceStart(const struct timeval *now) {
   struct timeval tv;
   if (!now)
     gettimeofday(&tv, NULL);
   else tv = *now;
 
-  return TIMEVAL_MSEC_SUBTRACT(tv, start_time);
+  return TIMEVAL_FSEC_SUBTRACT(tv, start_time);
+}
+
+// Convert a filename to a file:// URL. The return value must be freed.
+static char *filename_to_url(const char *filename) {
+  std::string url(filename);
+  char percent_buffer[10];
+
+#if WIN32
+  for (std::string::iterator p = url.begin(); p != url.end(); p++) {
+    if (*p == '\\')
+      *p = '/';
+  }
+  /* Put a pseudo-root directory before "C:/" or whatever. */
+  url = "/" + url;
+#endif
+
+  /* Percent-encode any troublesome characters. */
+  std::string::size_type i = 0;
+  /* See RFC 3986, section 3.3 "Path" for allowed characters. */
+  while ((i = url.find_first_of("?#[]%", i)) != std::string::npos) {
+    Snprintf(percent_buffer, sizeof(percent_buffer), "%%%02X", url[i]);
+    url.replace(i, 1, percent_buffer);
+    i += strlen(percent_buffer);
+  }
+
+  url = "file://" + url;
+
+  return strdup(url.c_str());
 }
 
 void NmapOps::Initialize() {
-  char tmpxsl[MAXPATHLEN];
-
   setaf(AF_INET);
 #if defined WIN32 || defined __amigaos__
   isr00t = 1;
@@ -189,22 +246,24 @@ void NmapOps::Initialize() {
   else
     isr00t = !(geteuid());
 #endif
+  have_pcap = true;
   debugging = 0;
   verbose = 0;
   min_packet_send_rate = 0.0; /* Unset. */
   max_packet_send_rate = 0.0; /* Unset. */
+  stats_interval = 0.0; /* Unset. */
   randomize_hosts = 0;
+  randomize_ports = 1;
   sendpref = PACKET_SEND_NOPREF;
   spoofsource = 0;
   fastscan = 0;
   device[0] = '\0';
-  interactivemode = 0;
   ping_group_sz = PING_GROUP_SZ;
   nogcc = 0;
   generate_random_ips = 0;
   reference_FPs = NULL;
   magic_port = 33000 + (get_random_uint() % 31000);
-  magic_port_set = 0;
+  magic_port_set = false;
   timing_level = 3;
   max_parallelism = 0;
   min_parallelism = 0;
@@ -217,6 +276,7 @@ void NmapOps::Initialize() {
   max_host_group_sz = 100000; // don't want to be restrictive unless user sets
   max_tcp_scan_delay = MAX_TCP_SCAN_DELAY;
   max_udp_scan_delay = MAX_UDP_SCAN_DELAY;
+  max_sctp_scan_delay = MAX_SCTP_SCAN_DELAY;
   max_ips_to_scan = 0;
   extra_payload_length = 0;
   extra_payload = NULL;
@@ -234,9 +294,12 @@ void NmapOps::Initialize() {
   override_excludeports = 0;
   version_intensity = 7;
   pingtype = PINGTYPE_UNKNOWN;
-  listscan = pingscan = allowall = ackscan = bouncescan = connectscan = 0;
-  rpcscan = nullscan = xmasscan = fragscan = synscan = windowscan = 0;
-  maimonscan = idlescan = finscan = udpscan = ipprotscan = noresolve = 0;
+  listscan = allowall = ackscan = bouncescan = connectscan = 0;
+  nullscan = xmasscan = fragscan = synscan = windowscan = 0;
+  maimonscan = idlescan = finscan = udpscan = ipprotscan = 0;
+  noportscan = noresolve = 0;
+  sctpinitscan = 0;
+  sctpcookieechoscan = 0;
   append_output = 0;
   memset(logfd, 0, sizeof(FILE *) * LOG_NUM_FILES);
   ttl = -1;
@@ -245,20 +308,21 @@ void NmapOps::Initialize() {
   gettimeofday(&start_time, NULL);
   pTrace = vTrace = false;
   reason = false;
+  adler32 = false;
   if (datadir) free(datadir);
   datadir = NULL;
-#if WIN32
-  Strncpy(tmpxsl, "nmap.xsl", sizeof(tmpxsl));
-#else
-  Snprintf(tmpxsl, sizeof(tmpxsl), "%s/nmap.xsl", NMAPDATADIR);
-#endif
+  xsl_stylesheet_set = false;
   if (xsl_stylesheet) free(xsl_stylesheet);
-  xsl_stylesheet = strdup(tmpxsl);
+  xsl_stylesheet = NULL;
   spoof_mac_set = false;
   mass_dns = true;
-  log_errors = false;
+  deprecated_xml_osclass = false;
   resolve_all = 0;
   dns_servers = NULL;
+  implicitARPPing = true;
+  numhosts_scanned = 0;
+  numhosts_up = 0;
+  numhosts_scanning = 0;
   noninteractive = false;
   current_scantype = STYPE_UNKNOWN;
   ipoptions = NULL;
@@ -269,12 +333,25 @@ void NmapOps::Initialize() {
   topportlevel = -1;
 #ifndef NOLUA
   script = 0;
+  scriptargs = NULL;
   scriptversion = 0;
   scripttrace = 0;
   scriptupdatedb = 0;
+  scripthelp = false;
+  chosenScripts.clear();
 #endif
   memset(&sourcesock, 0, sizeof(sourcesock));
   sourcesocklen = 0;
+  excludefd = NULL;
+  exclude_spec = NULL;
+  quashargv = 0;
+  inputfd = NULL;
+  idleProxy = NULL;
+  portlist = NULL;
+}
+
+bool NmapOps::SCTPScan() {
+  return sctpinitscan|sctpcookieechoscan;
 }
 
 bool NmapOps::TCPScan() {
@@ -285,14 +362,14 @@ bool NmapOps::UDPScan() {
   return udpscan;
 }
 
-  /* this function does not currently cover cases such as TCP SYN ping
-     scan which can go either way based on whether the user is root or
-     IPv6 is being used.  It will return false in those cases where a
-     RawScan is not neccessarily used. */
 bool NmapOps::RawScan() {
-  if (ackscan|finscan|idlescan|ipprotscan|maimonscan|nullscan|osscan|synscan|udpscan|windowscan|xmasscan)
+  if (ackscan||finscan||idlescan||ipprotscan||maimonscan||nullscan||osscan||synscan||udpscan||windowscan||xmasscan||sctpinitscan||sctpcookieechoscan||traceroute)
     return true;
-  if (pingtype & (PINGTYPE_ICMP_PING|PINGTYPE_ICMP_MASK|PINGTYPE_ICMP_TS|PINGTYPE_TCP_USE_ACK|PINGTYPE_UDP))
+  if (pingtype & (PINGTYPE_ICMP_PING|PINGTYPE_ICMP_MASK|PINGTYPE_ICMP_TS|PINGTYPE_TCP_USE_ACK|PINGTYPE_UDP|PINGTYPE_SCTP_INIT))
+    return true;
+  /* A SYN scan will only generate raw packets if nmap is running as root.
+     Otherwise, it becomes a connect scan. */
+  if ((pingtype & PINGTYPE_TCP_USE_SYN) && isr00t)
     return true;
 
    return false; 
@@ -300,27 +377,35 @@ bool NmapOps::RawScan() {
 
 
 void NmapOps::ValidateOptions() {
+	const char *privreq = "root privileges.";
 #ifdef WIN32
-	const char *privreq = "that WinPcap version 3.1 or higher and iphlpapi.dll be installed. You seem to be missing one or both of these.  Winpcap is available from http://www.winpcap.org.  iphlpapi.dll comes with Win98 and later operating sytems and NT 4.0 with SP4 or greater.  For previous windows versions, you may be able to take iphlpapi.dll from another system and place it in your system32 dir (e.g. c:\\windows\\system32)";
-#else
-	const char *privreq = "root privileges";
+	if (!have_pcap)
+		privreq = "WinPcap version 3.1 or higher and\n\
+iphlpapi.dll.  You seem to be missing one or both of these.  Winpcap is\n\
+available from http://www.winpcap.org.  iphlpapi.dll comes with Win98 and\n\
+later operating sytems and NT 4.0 with SP4 or greater.  For previous windows\n\
+versions, you may be able to take iphlpapi.dll from another system and place\n\
+it in your system32 dir (e.g. c:\\windows\\system32).\n\
+On Windows Vista and Windows 7, The WinPcap NPF service must be started by an\n\
+administrator before WinPcap can be used.  Running nmap.exe will open a UAC\n\
+dialog where you can start NPF if you have administrator privileges.";
 #endif
 
 
   /* Insure that at least one scantype is selected */
-  if (TCPScan() + UDPScan() + ipprotscan + listscan + pingscan == 0) {
-    if (isr00t && af() == AF_INET)
+  if (!noportscan && !(TCPScan() || UDPScan() || SCTPScan() || ipprotscan)) {
+    if (isr00t)
       synscan++;
     else connectscan++;
-    //    if (verbose) error("No tcp, udp, or ICMP scantype specified, assuming %s scan. Use -sP if you really don't want to portscan (and just want to see what hosts are up).", synscan? "SYN Stealth" : "vanilla tcp connect()");
+    //    if (verbose) error("No TCP, UDP, SCTP or ICMP scantype specified, assuming %s scan. Use -sn if you really don't want to portscan (and just want to see what hosts are up).", synscan? "SYN Stealth" : "vanilla tcp connect()");
   }
 
   if (pingtype != PINGTYPE_NONE && spoofsource) {
-    error("WARNING:  If -S is being used to fake your source address, you may also have to use -e <interface> and -PN .  If you are using it to specify your real source address, you can ignore this warning.");
+    error("WARNING: If -S is being used to fake your source address, you may also have to use -e <interface> and -Pn .  If you are using it to specify your real source address, you can ignore this warning.");
   }
 
   if (pingtype != PINGTYPE_NONE && idlescan) {
-    error("WARNING: Many people use -PN w/Idlescan to prevent pings from their true IP.  On the other hand, timing info Nmap gains from pings can allow for faster, more reliable scans.");
+    error("WARNING: Many people use -Pn w/Idlescan to prevent pings from their true IP.  On the other hand, timing info Nmap gains from pings can allow for faster, more reliable scans.");
     sleep(2); /* Give ppl a chance for ^C :) */
   }
 
@@ -329,31 +414,31 @@ void NmapOps::ValidateOptions() {
   }
 
  if (connectscan && spoofsource) {
-    error("WARNING:  -S will only affect the source address used in a connect() scan if you specify one of your own addresses.  Use -sS or another raw scan if you want to completely spoof your source address, but then you need to know what you're doing to obtain meaningful results.");
+    error("WARNING: -S will only affect the source address used in a connect() scan if you specify one of your own addresses.  Use -sS or another raw scan if you want to completely spoof your source address, but then you need to know what you're doing to obtain meaningful results.");
   }
 
- if ((pingtype & PINGTYPE_UDP) && (!isr00t || af() != AF_INET)) {
-   fatal("Sorry, UDP Ping (-PU) only works if you are root (because we need to read raw responses off the wire) and only for IPv4 (cause fyodor is too lazy right now to add IPv6 support and nobody has sent a patch)");
+ if ((pingtype & PINGTYPE_UDP) && (!isr00t)) {
+   fatal("Sorry, UDP Ping (-PU) only works if you are root (because we need to read raw responses off the wire)");
  }
 
- if ((pingtype & PINGTYPE_PROTO) && (!isr00t || af() != AF_INET)) {
-   fatal("Sorry, IPProto Ping (-PO) only works if you are root (because we need to read raw responses off the wire) and only for IPv4");
- }
-
- if (ipprotscan + (TCPScan() || UDPScan()) + listscan + pingscan > 1) {
-   fatal("Sorry, the IPProtoscan, Listscan, and Pingscan (-sO, -sL, -sP) must currently be used alone rather than combined with other scan types.");
- }
-
- if ((pingscan && pingtype == PINGTYPE_NONE)) {
-    fatal("-PN (skip ping) is incompatable with -sP (ping scan).  If you only want to enumerate hosts, try list scan (-sL)");
+ if ((pingtype & PINGTYPE_SCTP_INIT) && (!isr00t)) {
+   fatal("Sorry, SCTP INIT Ping (-PY) only works if you are root (because we need to read raw responses off the wire)");
   }
 
- if (pingscan && (TCPScan() || UDPScan() || ipprotscan || listscan)) {
-   fatal("Ping scan is not valid with any other scan types (the other ones all include a ping scan");
+ if ((pingtype & PINGTYPE_PROTO) && (!isr00t)) {
+   fatal("Sorry, IPProto Ping (-PO) only works if you are root (because we need to read raw responses off the wire)");
  }
 
- if (af() == AF_INET6 && (pingtype & (PINGTYPE_ICMP_PING|PINGTYPE_ICMP_MASK|PINGTYPE_ICMP_TS))) {
-   fatal("ICMP Echo, Timestamp and Address Mask pings are only valid for IPv4.");
+ if (ipprotscan && (TCPScan() || UDPScan() || SCTPScan())) {
+   fatal("Sorry, the IPProtoscan (-sO) must currently be used alone rather than combined with other scan types.");
+ }
+
+ if (noportscan && (TCPScan() || UDPScan() || SCTPScan() || ipprotscan)) {
+   fatal("-sL and -sn (skip port scan) are not valid with any other scan types");
+ }
+
+ if (af() == AF_INET6 && (pingtype & (PINGTYPE_ICMP_MASK|PINGTYPE_ICMP_TS))) {
+   fatal("ICMP Timestamp and Address Mask pings are only valid for IPv4.");
  }
 
  if (sendpref == PACKET_SEND_NOPREF) {
@@ -366,30 +451,26 @@ void NmapOps::ValidateOptions() {
 /* We start with stuff users should not do if they are not root */
   if (!isr00t) {
     
-    if (ackscan|finscan|idlescan|ipprotscan|maimonscan|nullscan|synscan|udpscan|windowscan|xmasscan) {
-      fatal("You requested a scan type which requires %s.", privreq);
+    if (ackscan|finscan|idlescan|ipprotscan|maimonscan|nullscan|synscan|udpscan|windowscan|xmasscan|sctpinitscan|sctpcookieechoscan) {
+      fatal("You requested a scan type which requires %s", privreq);
     }
     
     if (numdecoys > 0) {
-      fatal("Sorry, but decoys (-D) require %s.", privreq);
+      fatal("Sorry, but decoys (-D) require %s", privreq);
     }
     
     if (fragscan) {
-      fatal("Sorry, but fragscan requires %s.", privreq);
+      fatal("Sorry, but fragscan requires %s", privreq);
     }
     
     if (osscan) {
-      fatal("TCP/IP fingerprinting (for OS scan) requires %s.", privreq);
+      fatal("TCP/IP fingerprinting (for OS scan) requires %s", privreq);
     }
   }
   
   
-  if (numdecoys > 0 && rpcscan) {
-    error("WARNING:  RPC scan currently does not make use of decoys so don't count on that protection");
-  }
-  
   if (bouncescan && pingtype != PINGTYPE_NONE) 
-    log_write(LOG_STDOUT, "Hint: if your bounce scan target hosts aren't reachable from here, remember to use -PN so we don't try and ping them prior to the scan\n");
+    log_write(LOG_STDOUT, "Hint: if your bounce scan target hosts aren't reachable from here, remember to use -Pn so we don't try and ping them prior to the scan\n");
   
   if (ackscan+bouncescan+connectscan+finscan+idlescan+maimonscan+nullscan+synscan+windowscan+xmasscan > 1)
     fatal("You specified more than one type of TCP scan.  Please choose only one of -sA, -b, -sT, -sF, -sI, -sM, -sN, -sS, -sW, and -sX");
@@ -413,8 +494,8 @@ void NmapOps::ValidateOptions() {
   }
 #endif
   
-  if (osscan && pingscan) {
-    fatal("WARNING:  OS Scan is unreliable with a ping scan.  You need to use a scan type along with it, such as -sS, -sT, -sF, etc instead of -sP");
+  if (osscan && noportscan) {
+    fatal("WARNING: OS Scan is unreliable without a port scan.  You need to use a scan type along with it, such as -sS, -sT, -sF, etc instead of -sn");
   }
 
   if (osscan && ipprotscan) {
@@ -427,7 +508,7 @@ void NmapOps::ValidateOptions() {
     servicescan = 0;
   }
 
-  if (servicescan && pingscan)
+  if (servicescan && noportscan)
     servicescan = 0;
 
   if (defeat_rst_ratelimit && !synscan) {
@@ -438,7 +519,7 @@ void NmapOps::ValidateOptions() {
     resume_ip.s_addr = 0;
   
   if (magic_port_set && connectscan) {
-    error("WARNING:  -g is incompatible with the default connect() scan (-sT).  Use a raw scan such as -sS if you want to set the source port.");
+    error("WARNING: -g is incompatible with the default connect() scan (-sT).  Use a raw scan such as -sS if you want to set the source port.");
   }
 
   if (max_parallelism && min_parallelism && (min_parallelism > max_parallelism)) {
@@ -449,16 +530,10 @@ void NmapOps::ValidateOptions() {
     fatal("--min-rate=%g must be less than or equal to --max-rate=%g", min_packet_send_rate, max_packet_send_rate);
   }
   
-  if (af() == AF_INET6 && (numdecoys|osscan|bouncescan|fragscan|ackscan|finscan|idlescan|ipprotscan|maimonscan|nullscan|synscan|udpscan|windowscan|xmasscan)) {
-    fatal("Sorry -- IPv6 support is currently only available for connect() scan (-sT), ping scan (-sP), and list scan (-sL).  OS detection and decoys are also not supported with IPv6.  Further support is under consideration.");
+  if (af() == AF_INET6 && (generate_random_ips|numdecoys|bouncescan|fragscan|idlescan)) {
+    fatal("Sorry -- IPv6 support is currently only available for TCP, UDP, and SCTP port scans and list scan (-sL).  OS detection, random targets and decoys are also not supported with IPv6.  Further support is under consideration.");
   }
 
-  if (af() != AF_INET) mass_dns = false;
-
-  /* Prevent performance values from getting out of whack */
-  if (min_parallelism > max_parallelism)
-    max_parallelism = min_parallelism;
-
   if(ipoptions && osscan)
     error("WARNING: Ip options are NOT used while OS scanning!");
 
@@ -528,6 +603,33 @@ void NmapOps::setMaxHostGroupSz(unsigned int sz) {
 void NmapOps::setXSLStyleSheet(const char *xslname) {
   if (xsl_stylesheet) free(xsl_stylesheet);
   xsl_stylesheet = xslname? strdup(xslname) : NULL;
+  xsl_stylesheet_set = true;
+}
+
+/* Returns the full path or URL that should be printed in the XML
+   output xml-stylesheet element.  Returns NULL if the whole element
+   should be skipped */
+char *NmapOps::XSLStyleSheet() {
+  char tmpxsl[MAXPATHLEN];
+
+  if (xsl_stylesheet_set)
+    return xsl_stylesheet;
+
+  if (nmap_fetchfile(tmpxsl, sizeof(tmpxsl), "nmap.xsl") == 1) {
+    xsl_stylesheet = filename_to_url(tmpxsl);
+  } else {
+#if WIN32
+    /* Use a relative URL on Windows if nmap_fetchfile failed. It won't work,
+       but it gives a clue that there is an nmap.xsl somewhere. */
+    Strncpy(tmpxsl, "nmap.xsl", sizeof(tmpxsl));
+    xsl_stylesheet = strdup(tmpxsl);
+#else
+    Snprintf(tmpxsl, sizeof(tmpxsl), "%s/nmap.xsl", NMAPDATADIR);
+    xsl_stylesheet = filename_to_url(tmpxsl);
+#endif
+  }
+
+  return xsl_stylesheet;
 }
 
 void NmapOps::setSpoofMACAddress(u8 *mac_data) {
@@ -537,12 +639,17 @@ void NmapOps::setSpoofMACAddress(u8 *mac_data) {
 
 #ifndef NOLUA
 void NmapOps::chooseScripts(char* argument) {
-	char *ap;
+	char *p;
 
-	ap = strtok(argument, ",");
-	while(ap != NULL) {
-		chosenScripts.push_back(std::string(ap));
-		ap = strtok(NULL, ",");
+	for (;;) {
+		p = strchr(argument, ',');
+		if (p == NULL) {
+			chosenScripts.push_back(std::string(argument));
+			break;
+		} else {
+			chosenScripts.push_back(std::string(argument, p - argument));
+			argument = p + 1;
+		}
 	}
 }
 #endif
