@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -15,11 +15,12 @@
  * technology into proprietary software, we sell alternative licenses      *
  * (contact sales@insecure.com).  Dozens of software vendors already       *
  * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * detection, version detection, and the Nmap Scripting Engine.            *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
  * "derivative work" for the purpose of this license if it does any of the *
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
@@ -27,40 +28,32 @@
  *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                * 
+ *   not derivative works.)                                                *
  * o Integrates/includes/aggregates Nmap into a proprietary executable     *
  *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is just meant to        *
- * clarify our interpretation of derived works with some common examples.  *
- * These restrictions only apply when you actually redistribute Nmap.  For *
- * example, nothing stops you from writing and selling a proprietary       *
- * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://nmap.org to download Nmap.                                       *
- *                                                                         *
- * We don't consider these to be added restrictions on top of the GPL, but *
- * just a clarification of how we interpret "derived works" as it applies  *
- * to our GPL-licensed Nmap product.  This is similar to the way Linus     *
- * Torvalds has announced his interpretation of how "derived works"        *
- * applies to Linux kernel modules.  Our interpretation refers only to     *
- * Nmap - we don't speak for any other GPL products.                       *
+ * works of Nmap, as well as other software we distribute under this       *
+ * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+ * but is meant to clarify our interpretation of derived works with some   *
+ * common examples.  Our interpretation applies only to Nmap--we don't     *
+ * speak for other people's GPL works.                                     *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included COPYING.OpenSSL file, and distribute linked      *
- * combinations including the two. You must obey the GNU GPL in all        *
+ * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+ * linked combinations including the two. You must obey the GNU GPL in all *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
  * but you are not obligated to do so.                                     *
@@ -76,17 +69,18 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to fyodor@insecure.org for possible incorporation into the main         *
+ * to nmap-dev@insecure.org for possible incorporation into the main       *
  * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
- * to reuse, modify, and relicense the code.  Nmap will always be          *
- * available Open Source, but this is important because the inability to   *
- * relicense code has caused devastating problems for other Free Software  *
- * projects (such as KDE and NASM).  We also occasionally relicense the    *
- * code to third parties as discussed above.  If you wish to specify       *
- * special license conditions of your contributions, just say so when you  *
- * send them.                                                              *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -97,7 +91,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: NmapOps.h 9191 2008-07-30 00:15:57Z david $ */
+/* $Id: NmapOps.h 29688 2012-08-28 14:29:11Z david $ */
 
 #include "nmap.h"
 #include "global_structures.h"
@@ -116,28 +110,29 @@ class NmapOps {
   /* Returns 0 for success, nonzero if no source has been set or any other
      failure */
   int SourceSockAddr(struct sockaddr_storage *ss, size_t *ss_len);
+  /* Returns a const pointer to the source address if set, or NULL if unset. */
+  const struct sockaddr_storage *SourceSockAddr() const;
   /* Note that it is OK to pass in a sockaddr_in or sockaddr_in6 casted
      to sockaddr_storage */
   void setSourceSockAddr(struct sockaddr_storage *ss, size_t ss_len);
 
 // The time this obj. was instantiated   or last ReInit()ed.
   const struct timeval *getStartTime() { return &start_time; }
-  // Number of milliseconds since getStartTime().  The current time is an
+  // Number of seconds since getStartTime().  The current time is an
   // optional argument to avoid an extra gettimeofday() call.
-  int TimeSinceStartMS(struct timeval *now=NULL); 
-  struct in_addr v4source();
-  const struct in_addr *v4sourceip();
+  float TimeSinceStart(const struct timeval *now=NULL);
 
 
 
   bool TCPScan(); /* Returns true if at least one chosen scan type is TCP */
   bool UDPScan(); /* Returns true if at least one chosen scan type is UDP */
+  bool SCTPScan(); /* Returns true if at least one chosen scan type is SCTP */
 
   /* Returns true if at least one chosen scan type uses raw packets.
      It does not currently cover cases such as TCP SYN ping scan which
      can go either way based on whether the user is root or IPv6 is
      being used.  It will return false in those cases where a RawScan
-     is not neccessarily used. */
+     is not necessarily used. */
   bool RawScan();
   void ValidateOptions(); /* Checks that the options given are
                              reasonable and consistant.  If they aren't, the
@@ -145,6 +140,8 @@ class NmapOps {
 		             adjustments (quietly or with a warning to the
 		             user). */
   int isr00t;
+  /* Whether we have pcap functions (can be false on Windows). */
+  bool have_pcap;
   int debugging;
 
 #define PACKET_SEND_NOPREF 1
@@ -182,17 +179,20 @@ class NmapOps {
   float min_packet_send_rate;
   /* The requested maximum packet sending rate, or 0.0 if unset. */
   float max_packet_send_rate;
+  /* The requested auto stats printing interval, or 0.0 if unset. */
+  float stats_interval;
   int randomize_hosts;
+  int randomize_ports;
   int spoofsource; /* -S used */
   int fastscan;
   char device[64];
-  int interactivemode;
   int ping_group_sz;
   int nogcc; /* Turn off group congestion control with --nogcc */
   int generate_random_ips; /* -iR option */
   FingerPrintDB *reference_FPs; /* Used in the new OS scan system. */
-  u16 magic_port;
-  unsigned short magic_port_set; /* Was this set by user? */
+  std::vector<FingerMatch> os_labels_ipv6;
+  u16 magic_port; /* The source port set by -g or --source-port. */
+  bool magic_port_set; /* Was this set by user? */
 
   /* Scan timing/politeness issues */
   int timing_level; // 0-5, corresponding to Paranoid, Sneaky, Polite, Normal, Aggressive, Insane
@@ -218,7 +218,7 @@ class NmapOps {
   void setMinRttTimeout(int rtt);
   void setInitialRttTimeout(int rtt);
   void setMaxRetransmissions(int max_retransmit);
-  int getMaxRetransmissions() { return max_retransmissions; }
+  unsigned int getMaxRetransmissions() { return max_retransmissions; }
 
   /* Similar functions for Host group size */
   int minHostGroupSz() { return min_host_group_sz; }
@@ -227,8 +227,10 @@ class NmapOps {
   void setMaxHostGroupSz(unsigned int sz);
   unsigned int maxTCPScanDelay() { return max_tcp_scan_delay; }
   unsigned int maxUDPScanDelay() { return max_udp_scan_delay; }
+  unsigned int maxSCTPScanDelay() { return max_sctp_scan_delay; }
   void setMaxTCPScanDelay(unsigned int delayMS) { max_tcp_scan_delay = delayMS; }
   void setMaxUDPScanDelay(unsigned int delayMS) { max_udp_scan_delay = delayMS; }
+  void setMaxSCTPScanDelay(unsigned int delayMS) { max_sctp_scan_delay = delayMS; }
 
   /* Sets the Name of the XML stylesheet to be printed in XML output.
      If this is never called, a default stylesheet distributed with
@@ -238,7 +240,7 @@ class NmapOps {
   /* Returns the full path or URL that should be printed in the XML
      output xml-stylesheet element.  Returns NULL if the whole element
      should be skipped */
-  char *XSLStyleSheet() { return xsl_stylesheet; }
+  char *XSLStyleSheet();
 
   /* Sets the spoofed MAC address */
   void setSpoofMACAddress(u8 *mac_data);
@@ -282,7 +284,6 @@ class NmapOps {
   int servicescan;
   int pingtype;
   int listscan;
-  int pingscan;
   int allowall;
   int fragscan; /* 0 or MTU (without IPv4 header size) */
   int ackscan;
@@ -290,15 +291,18 @@ class NmapOps {
   int connectscan;
   int finscan;
   int idlescan;
+  char* idleProxy; /* The idle host used to "Proxy" an idle scan */
   int ipprotscan;
   int maimonscan;
   int nullscan;
-  int rpcscan;
   int synscan;
   int udpscan;
+  int sctpinitscan;
+  int sctpcookieechoscan;
   int windowscan;
   int xmasscan;
   int noresolve;
+  int noportscan;
   int append_output; /* Append to any output files rather than overwrite */
   FILE *logfd[LOG_NUM_FILES];
   FILE *nmap_stdout; /* Nmap standard output */
@@ -316,16 +320,36 @@ class NmapOps {
   bool mass_dns;
   int resolve_all;
   char *dns_servers;
-  bool log_errors;
+
+  /* Do IPv4 ARP or IPv6 ND scan of directly connected Ethernet hosts, even if
+     non-ARP host discovery options are used? This is normally more efficient,
+     not only because ARP/ND scan is faster, but because we need the MAC
+     addresses provided by ARP or ND scan in order to do IP-based host discovery
+     anyway. But when a network uses proxy ARP, all hosts will appear to be up
+     unless you do an IP host discovery on them. This option is true by default. */
+  bool implicitARPPing;
+
+  // If true, write <os><osclass/><osmatch/></os> as in xmloutputversion 1.03
+  // rather than <os><osmatch><osclass/></osmatch></os> as in 1.04 and later.
+  bool deprecated_xml_osclass;
+
   bool traceroute;
   bool reason;
+  bool adler32;
+  FILE *excludefd;
+  char *exclude_spec;
+  int quashargv;
+  FILE *inputfd;
+  char *portlist; /* Ports list specified by user */
 
 #ifndef NOLUA
   int script;
   char *scriptargs;
+  char *scriptargsfile;
   int scriptversion;
   int scripttrace;
   int scriptupdatedb;
+  bool scripthelp;
   void chooseScripts(char* argument);
   std::vector<std::string> chosenScripts;
 #endif
@@ -349,9 +373,10 @@ class NmapOps {
   int max_rtt_timeout;
   int min_rtt_timeout;
   int initial_rtt_timeout;
-  int max_retransmissions;
+  unsigned int max_retransmissions;
   unsigned int max_tcp_scan_delay;
   unsigned int max_udp_scan_delay;
+  unsigned int max_sctp_scan_delay;
   unsigned int min_host_group_sz;
   unsigned int max_host_group_sz;
   void Initialize();
@@ -361,6 +386,7 @@ class NmapOps {
   struct timeval start_time;
   bool pTrace; // Whether packet tracing has been enabled
   bool vTrace; // Whether version tracing has been enabled
+  bool xsl_stylesheet_set;
   char *xsl_stylesheet;
   u8 spoof_mac[6];
   bool spoof_mac_set;
