@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2002 Dug Song <dugsong@monkey.org>
  *
- * $Id: addr-util.c,v 1.4 2005/01/23 07:36:54 dugsong Exp $
+ * $Id: addr-util.c 539 2005-01-23 07:36:54Z dugsong $
  */
 
 #ifdef _WIN32
@@ -174,6 +174,7 @@ ip_pton(const char *p, ip_addr_t *ip)
 char *
 ip6_ntop(const ip6_addr_t *ip6, char *dst, size_t len)
 {
+	uint16_t data[IP6_ADDR_LEN / 2];
 	struct { int base, len; } best, cur;
 	char *p = dst;
 	int i;
@@ -183,12 +184,18 @@ ip6_ntop(const ip6_addr_t *ip6, char *dst, size_t len)
 	if (len < 46)
 		return (NULL);
 	
+	/* Copy into 16-bit array. */
+	for (i = 0; i < IP6_ADDR_LEN / 2; i++) {
+		data[i] = ip6->data[2 * i] << 8;
+		data[i] |= ip6->data[2 * i + 1];
+	}
+	
 	best.base = cur.base = -1;
 	/*
 	 * Algorithm borrowed from Vixie's inet_pton6()
 	 */
 	for (i = 0; i < IP6_ADDR_LEN; i += 2) {
-		if (*((uint16_t *)&ip6->data[i]) == 0) {
+		if (data[i / 2] == 0) {
 			if (cur.base == -1) {
 				cur.base = i;
 				cur.len = 0;
@@ -215,13 +222,13 @@ ip6_ntop(const ip6_addr_t *ip6, char *dst, size_t len)
 			i += best.len;
 		} else if (i == 12 && best.base == 0 &&
 		    (best.len == 10 || (best.len == 8 &&
-			*((uint16_t *)&ip6->data[10]) == 0xffff))) {
-			if (ip_ntop((ip_addr_t *)&ip6->data[12], p,
+			data[5] == 0xffff))) {
+			if (ip_ntop((ip_addr_t *)&data[6], p,
 			    len - (p - dst)) == NULL)
 				return (NULL);
 			return (dst);
 		} else p += sprintf(p, "%x:",
-		    ntohs(*((uint16_t *)&ip6->data[i])));
+		    ntohs(data[i / 2]));
 	}
 	if (best.base + 2 + best.len == IP6_ADDR_LEN) {
 		*p = '\0';
