@@ -3,7 +3,7 @@
  *
  * Copyright (c) 2001 Dug Song <dugsong@monkey.org>
  *
- * $Id: intf.c,v 1.55 2005/02/10 16:57:35 dugsong Exp $
+ * $Id: intf.c 616 2006-01-09 07:09:49Z dugsong $
  */
 
 #ifdef _WIN32
@@ -33,6 +33,10 @@
 # include <netinet/in.h>
 # include <netinet/in_var.h>
 #endif
+#ifdef HAVE_NETINET_IN6_VAR_H
+# include <sys/protosw.h>
+# include <netinet/in6_var.h>
+#endif
 
 #include <errno.h>
 #include <stdio.h>
@@ -60,13 +64,16 @@
 #endif
 
 #ifdef HAVE_SOCKADDR_SA_LEN
-# define NEXTIFR(i)	((struct ifreq *)((u_char *)&i->ifr_addr + \
-				(i->ifr_addr.sa_len ? i->ifr_addr.sa_len : \
-				 sizeof(i->ifr_addr))))
+# define max(a, b) ((a) > (b) ? (a) : (b))
+# define NEXTIFR(i)	((struct ifreq *) \
+				max((u_char *)i + sizeof(struct ifreq), \
+				(u_char *)&i->ifr_addr + i->ifr_addr.sa_len))
 #else
 # define NEXTIFR(i)	(i + 1)
 #endif
 
+#define NEXTLIFR(i)	(i + 1)
+
 /* XXX - superset of ifreq, for portable SIOC{A,D}IFADDR */
 struct dnet_ifaliasreq {
 	char		ifra_name[IFNAMSIZ];
@@ -80,6 +87,9 @@ struct intf_handle {
 	int		fd;
 	int		fd6;
 	struct ifconf	ifc;
+#ifdef SIOCGLIFCONF
+	struct lifconf	lifc;
+#endif
 	u_char		ifcbuf[4192];
 };
 
@@ -99,7 +109,7 @@ intf_flags_to_iff(u_short flags, int iff)
 }
 
 static u_int
-intf_iff_to_flags(int iff)
+intf_iff_to_flags(uint64_t iff)
 {
 	u_int n = 0;
 
@@ -115,6 +125,12 @@ intf_iff_to_flags(int iff)
 		n |= INTF_FLAG_BROADCAST;
 	if (iff & IFF_MULTICAST)
 		n |= INTF_FLAG_MULTICAST;
+#ifdef IFF_IPMP
+	/* Unset the BROADCAST and MULTICAST flags from Solaris IPMP interfaces,
+	 * otherwise _intf_set_type will think they are INTF_TYPE_ETH. */
+	if (iff & IFF_IPMP)
+		n &= ~(INTF_FLAG_BROADCAST | INTF_FLAG_MULTICAST);
+#endif
 
 	return (n);
 }
@@ -124,16 +140,17 @@ intf_open(void)
 {
 	intf_t *intf;
 	int one = 1;
-
+	
 	if ((intf = calloc(1, sizeof(*intf))) != NULL) {
 		intf->fd = intf->fd6 = -1;
 		
 		if ((intf->fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
 			return (intf_close(intf));
 
-		setsockopt(intf->fd, SOL_SOCKET, SO_BROADCAST, 
-			   (const char *) &one, sizeof(one));
-#ifdef SIOCGIFNETMASK_IN6
+		setsockopt(intf->fd, SOL_SOCKET, SO_BROADCAST,
+			(const char *) &one, sizeof(one));
+
+#if defined(SIOCGLIFCONF) || defined(SIOCGIFNETMASK_IN6) || defined(SIOCGIFNETMASK6)
 		if ((intf->fd6 = socket(AF_INET6, SOCK_DGRAM, 0)) < 0) {
 #  ifdef EPROTONOSUPPORT
 			if (errno != EPROTONOSUPPORT)
@@ -373,23 +390,95 @@ intf_set(intf_t *intf, const struct intf_entry *entry)
 static void
 _intf_set_type(struct intf_entry *entry)
 {
-	if ((entry->intf_flags & INTF_FLAG_BROADCAST) != 0)
+	if ((entry->intf_flags & INTF_FLAG_LOOPBACK) != 0)
+		entry->intf_type = INTF_TYPE_LOOPBACK;
+	else if ((entry->intf_flags & INTF_FLAG_BROADCAST) != 0)
 		entry->intf_type = INTF_TYPE_ETH;
 	else if ((entry->intf_flags & INTF_FLAG_POINTOPOINT) != 0)
 		entry->intf_type = INTF_TYPE_TUN;
-	else if ((entry->intf_flags & INTF_FLAG_LOOPBACK) != 0)
-		entry->intf_type = INTF_TYPE_LOOPBACK;
 	else
 		entry->intf_type = INTF_TYPE_OTHER;
 }
 
+#ifdef SIOCGLIFCONF
+int
+_intf_get_noalias(intf_t *intf, struct intf_entry *entry)
+{
+	struct lifreq lifr;
+	int fd;
+
+	/* Get interface index. */
+	entry->intf_index = if_nametoindex(entry->intf_name);
+	if (entry->intf_index == 0)
+		return (-1);
+
+	strlcpy(lifr.lifr_name, entry->intf_name, sizeof(lifr.lifr_name));
+
+	/* Get interface flags. Here he also check whether we need to use fd or
+	 * fd6 in the rest of the function. Using the wrong address family in
+	 * the ioctls gives ENXIO on Solaris. */
+	if (ioctl(intf->fd, SIOCGLIFFLAGS, &lifr) >= 0)
+		fd = intf->fd;
+	else if (intf->fd6 != -1 && ioctl(intf->fd6, SIOCGLIFFLAGS, &lifr) >= 0)
+		fd = intf->fd6;
+	else
+		return (-1);
+	
+	entry->intf_flags = intf_iff_to_flags(lifr.lifr_flags);
+	_intf_set_type(entry);
+	
+	/* Get interface MTU. */
+#ifdef SIOCGLIFMTU
+	if (ioctl(fd, SIOCGLIFMTU, &lifr) < 0)
+#endif
+		return (-1);
+	entry->intf_mtu = lifr.lifr_mtu;
+
+	entry->intf_addr.addr_type = entry->intf_dst_addr.addr_type =
+	    entry->intf_link_addr.addr_type = ADDR_TYPE_NONE;
+	
+	/* Get primary interface address. */
+	if (ioctl(fd, SIOCGLIFADDR, &lifr) == 0) {
+		addr_ston((struct sockaddr *)&lifr.lifr_addr, &entry->intf_addr);
+		if (ioctl(fd, SIOCGLIFNETMASK, &lifr) < 0)
+			return (-1);
+		addr_stob((struct sockaddr *)&lifr.lifr_addr, &entry->intf_addr.addr_bits);
+	}
+	/* Get other addresses. */
+	if (entry->intf_type == INTF_TYPE_TUN) {
+		if (ioctl(fd, SIOCGLIFDSTADDR, &lifr) == 0) {
+			if (addr_ston((struct sockaddr *)&lifr.lifr_addr,
+			    &entry->intf_dst_addr) < 0)
+				return (-1);
+		}
+	} else if (entry->intf_type == INTF_TYPE_ETH) {
+		eth_t *eth;
+		
+		if ((eth = eth_open(entry->intf_name)) != NULL) {
+			if (!eth_get(eth, &entry->intf_link_addr.addr_eth)) {
+				entry->intf_link_addr.addr_type =
+				    ADDR_TYPE_ETH;
+				entry->intf_link_addr.addr_bits =
+				    ETH_ADDR_BITS;
+			}
+			eth_close(eth);
+		}
+	}
+	return (0);
+}
+#else
 static int
 _intf_get_noalias(intf_t *intf, struct intf_entry *entry)
 {
 	struct ifreq ifr;
 
+	/* Get interface index. */
+	entry->intf_index = if_nametoindex(entry->intf_name);
+	if (entry->intf_index == 0)
+		return (-1);
+
 	strlcpy(ifr.ifr_name, entry->intf_name, sizeof(ifr.ifr_name));
-	
+
 	/* Get interface flags. */
 	if (ioctl(intf->fd, SIOCGIFFLAGS, &ifr) < 0)
 		return (-1);
@@ -451,6 +540,7 @@ _intf_get_noalias(intf_t *intf, struct intf_entry *entry)
 	}
 	return (0);
 }
+#endif
 
 #ifdef SIOCLIFADDR
 /* XXX - aliases on IRIX don't show up in SIOCGIFCONF */
@@ -479,6 +569,71 @@ _intf_get_aliases(intf_t *intf, struct intf_entry *entry)
 	
 	return (0);
 }
+#elif defined(SIOCGLIFCONF)
+static int
+_intf_get_aliases(intf_t *intf, struct intf_entry *entry)
+{
+	struct lifreq *lifr, *llifr;
+	struct lifreq tmplifr;
+	struct addr *ap, *lap;
+	char *p;
+	
+	if (intf->lifc.lifc_len < (int)sizeof(*lifr)) {
+		errno = EINVAL;
+		return (-1);
+	}
+	entry->intf_alias_num = 0;
+	ap = entry->intf_alias_addrs;
+	llifr = (struct lifreq *)intf->lifc.lifc_buf + 
+	    (intf->lifc.lifc_len / sizeof(*llifr));
+	lap = (struct addr *)((u_char *)entry + entry->intf_len);
+	
+	/* Get addresses for this interface. */
+	for (lifr = intf->lifc.lifc_req; lifr < llifr && (ap + 1) < lap;
+	    lifr = NEXTLIFR(lifr)) {
+		/* XXX - Linux, Solaris ifaliases */
+		if ((p = strchr(lifr->lifr_name, ':')) != NULL)
+			*p = '\0';
+		
+		if (strcmp(lifr->lifr_name, entry->intf_name) != 0) {
+			if (p) *p = ':';
+			continue;
+		}
+		
+		/* Fix the name back up */
+		if (p) *p = ':';
+
+		if (addr_ston((struct sockaddr *)&lifr->lifr_addr, ap) < 0)
+			continue;
+		
+		/* XXX */
+		if (ap->addr_type == ADDR_TYPE_ETH) {
+			memcpy(&entry->intf_link_addr, ap, sizeof(*ap));
+			continue;
+		} else if (ap->addr_type == ADDR_TYPE_IP) {
+			if (ap->addr_ip == entry->intf_addr.addr_ip ||
+			    ap->addr_ip == entry->intf_dst_addr.addr_ip)
+				continue;
+			strlcpy(tmplifr.lifr_name, lifr->lifr_name, sizeof(tmplifr.lifr_name));
+			if (ioctl(intf->fd, SIOCGIFNETMASK, &tmplifr) == 0)
+				addr_stob((struct sockaddr *)&tmplifr.lifr_addr, &ap->addr_bits);
+		} else if (ap->addr_type == ADDR_TYPE_IP6 && intf->fd6 != -1) {
+			if (memcmp(&ap->addr_ip6, &entry->intf_addr.addr_ip6, IP6_ADDR_LEN) == 0 ||
+			    memcmp(&ap->addr_ip6, &entry->intf_dst_addr.addr_ip6, IP6_ADDR_LEN) == 0)
+				continue;
+			strlcpy(tmplifr.lifr_name, lifr->lifr_name, sizeof(tmplifr.lifr_name));
+			if (ioctl(intf->fd6, SIOCGLIFNETMASK, &tmplifr) == 0) {
+				addr_stob((struct sockaddr *)&tmplifr.lifr_addr,
+				    &ap->addr_bits);
+			}
+			else perror("SIOCGLIFNETMASK");
+		}
+		ap++, entry->intf_alias_num++;
+	}
+	entry->intf_len = (u_char *)ap - (u_char *)entry;
+	
+	return (0);
+}
 #else
 static int
 _intf_get_aliases(intf_t *intf, struct intf_entry *entry)
@@ -506,14 +661,16 @@ _intf_get_aliases(intf_t *intf, struct intf_entry *entry)
 			*p = '\0';
 		
 		if (strcmp(ifr->ifr_name, entry->intf_name) != 0) {
-		  if (p) *p = ':';
+			if (p) *p = ':';
 			continue;
 		}
 		
-		if (p) *p = ':'; /* Fix the name back up */
+		/* Fix the name back up */
+		if (p) *p = ':';
+
 		if (addr_ston(&ifr->ifr_addr, ap) < 0)
 			continue;
-				
+		
 		/* XXX */
 		if (ap->addr_type == ADDR_TYPE_ETH) {
 			memcpy(&entry->intf_link_addr, ap, sizeof(*ap));
@@ -522,11 +679,9 @@ _intf_get_aliases(intf_t *intf, struct intf_entry *entry)
 			if (ap->addr_ip == entry->intf_addr.addr_ip ||
 			    ap->addr_ip == entry->intf_dst_addr.addr_ip)
 				continue;
-			strlcpy(tmpifr.ifr_name, ifr->ifr_name, 
-			       sizeof(tmpifr.ifr_name));
+			strlcpy(tmpifr.ifr_name, ifr->ifr_name, sizeof(tmpifr.ifr_name));
 			if (ioctl(intf->fd, SIOCGIFNETMASK, &tmpifr) == 0)
-			  addr_stob(&tmpifr.ifr_addr, &ap->addr_bits);
-
+				addr_stob(&tmpifr.ifr_addr, &ap->addr_bits);
 		}
 #ifdef SIOCGIFNETMASK_IN6
 		else if (ap->addr_type == ADDR_TYPE_IP6 && intf->fd6 != -1) {
@@ -541,9 +696,50 @@ _intf_get_aliases(intf_t *intf, struct intf_entry *entry)
 			}
 			else perror("SIOCGIFNETMASK_IN6");
 		}
+#else
+#ifdef SIOCGIFNETMASK6
+		else if (ap->addr_type == ADDR_TYPE_IP6 && intf->fd6 != -1) {
+			struct in6_ifreq ifr6;
+
+			/* XXX - sizeof(ifr) < sizeof(ifr6) */
+			memcpy(&ifr6, ifr, sizeof(ifr6));
+			
+			if (ioctl(intf->fd6, SIOCGIFNETMASK6, &ifr6) == 0) {
+				/* For some reason this is 0 after the ioctl. */
+				ifr6.ifr_Addr.sin6_family = AF_INET6;
+				addr_stob((struct sockaddr *)&ifr6.ifr_Addr,
+				    &ap->addr_bits);
+			}
+			else perror("SIOCGIFNETMASK6");
+		}
+#endif
 #endif
 		ap++, entry->intf_alias_num++;
 	}
+#ifdef HAVE_LINUX_PROCFS
+#define PROC_INET6_FILE	"/proc/net/if_inet6"
+	{
+		FILE *f;
+		char buf[256], s[8][5], name[INTF_NAME_LEN];
+		u_int idx, bits, scope, flags;
+		
+		if ((f = fopen(PROC_INET6_FILE, "r")) != NULL) {
+			while (ap < lap &&
+			       fgets(buf, sizeof(buf), f) != NULL) {
+				sscanf(buf, "%04s%04s%04s%04s%04s%04s%04s%04s %x %02x %02x %02x %32s\n",
+				    s[0], s[1], s[2], s[3], s[4], s[5], s[6], s[7],
+				    &idx, &bits, &scope, &flags, name);
+				if (strcmp(name, entry->intf_name) == 0) {
+					snprintf(buf, sizeof(buf), "%s:%s:%s:%s:%s:%s:%s:%s/%d",
+					    s[0], s[1], s[2], s[3], s[4], s[5], s[6], s[7], bits);
+					addr_aton(buf, ap);
+					ap++, entry->intf_alias_num++;
+				}
+			}
+			fclose(f);
+		}
+	}
+#endif
 	entry->intf_len = (u_char *)ap - (u_char *)entry;
 	
 	return (0);
@@ -565,31 +761,45 @@ intf_get(intf_t *intf, struct intf_entry *entry)
 	return (_intf_get_aliases(intf, entry));
 }
 
+/* Look up an interface from an index, such as a sockaddr_in6.sin6_scope_id. */
+int
+intf_get_index(intf_t *intf, struct intf_entry *entry, int af, unsigned int index)
+{
+	char namebuf[IFNAMSIZ];
+	char *devname;
+
+	/* af is ignored; only used in intf-win32.c. */
+	devname = if_indextoname(index, namebuf);
+	if (devname == NULL)
+		return (-1);
+	strlcpy(entry->intf_name, devname, sizeof(entry->intf_name));
+	return intf_get(intf, entry);
+}
+
 static int
 _match_intf_src(const struct intf_entry *entry, void *arg)
 {
-	int matched = 0;
-	int cnt;
 	struct intf_entry *save = (struct intf_entry *)arg;
+	int matched = 0, cnt;
 	
-	if (entry->intf_addr.addr_type == ADDR_TYPE_IP && 
+	if (entry->intf_addr.addr_type == ADDR_TYPE_IP &&
 	    entry->intf_addr.addr_ip == save->intf_addr.addr_ip)
-	  matched = 1;
-  
+		matched = 1;
+
 	for (cnt = 0; !matched && cnt < (int) entry->intf_alias_num; cnt++) {
-	  if (entry->intf_alias_addrs[cnt].addr_type != ADDR_TYPE_IP)
-	    continue;
-	  if (entry->intf_alias_addrs[cnt].addr_ip == save->intf_addr.addr_ip)
-	    matched = 1;
+		if (entry->intf_alias_addrs[cnt].addr_type != ADDR_TYPE_IP)
+			continue;
+		if (entry->intf_alias_addrs[cnt].addr_ip == save->intf_addr.addr_ip)
+			matched = 1;
 	}
 
 	if (matched) {
-	  /* XXX - truncated result if entry is too small. */
-	  if (save->intf_len < entry->intf_len)
-	    memcpy(save, entry, save->intf_len);
-	  else
-	    memcpy(save, entry, entry->intf_len);
-	  return (1);
+		/* XXX - truncated result if entry is too small. */
+		if (save->intf_len < entry->intf_len)
+			memcpy(save, entry, save->intf_len);
+		else
+			memcpy(save, entry, entry->intf_len);
+		return (1);
 	}
 	return (0);
 }
@@ -610,7 +820,7 @@ int
 intf_get_dst(intf_t *intf, struct intf_entry *entry, struct addr *dst)
 {
 	struct sockaddr_in sin;
-	int n;
+	socklen_t n;
 	
 	if (dst->addr_type != ADDR_TYPE_IP) {
 		errno = EINVAL;
@@ -653,8 +863,10 @@ intf_loop(intf_t *intf, intf_handler callback, void *arg)
 	intf->ifc.ifc_buf = (caddr_t)intf->ifcbuf;
 	intf->ifc.ifc_len = sizeof(intf->ifcbuf);
 	
-	if (ioctl(intf->fd, SIOCGIFCONF, &intf->ifc) < 0)
+	if (ioctl(intf->fd, SIOCGIFCONF, &intf->ifc) < 0) {
+		fclose(fp);
 		return (-1);
+	}
 
 	ret = 0;
 	while (fgets(buf, sizeof(buf), fp) != NULL) {
@@ -686,6 +898,79 @@ intf_loop(intf_t *intf, intf_handler callback, void *arg)
 	
 	return (ret);
 }
+#elif defined(SIOCGLIFCONF)
+int
+intf_loop(intf_t *intf, intf_handler callback, void *arg)
+{
+	struct intf_entry *entry;
+	struct lifreq *lifr, *llifr, *plifr;
+	char *p, ebuf[BUFSIZ];
+	int ret;
+
+	entry = (struct intf_entry *)ebuf;
+
+	/* http://www.unix.com/man-page/opensolaris/7p/if_tcp */
+	intf->lifc.lifc_family = AF_UNSPEC;
+	intf->lifc.lifc_flags = 0;
+#ifdef LIFC_UNDER_IPMP
+	intf->lifc.lifc_flags |= LIFC_UNDER_IPMP;
+#endif
+	intf->lifc.lifc_buf = (caddr_t)intf->ifcbuf;
+	intf->lifc.lifc_len = sizeof(intf->ifcbuf);
+	
+	if (ioctl(intf->fd, SIOCGLIFCONF, &intf->lifc) < 0)
+		return (-1);
+
+	llifr = (struct lifreq *)&intf->lifc.lifc_buf[intf->lifc.lifc_len];
+	
+	for (lifr = intf->lifc.lifc_req; lifr < llifr; lifr = NEXTLIFR(lifr)) {
+		/* XXX - Linux, Solaris ifaliases */
+		if ((p = strchr(lifr->lifr_name, ':')) != NULL)
+			*p = '\0';
+		
+		for (plifr = intf->lifc.lifc_req; plifr < lifr; plifr = NEXTLIFR(lifr)) {
+			if (strcmp(lifr->lifr_name, plifr->lifr_name) == 0)
+				break;
+		}
+		if (lifr > intf->lifc.lifc_req && plifr < lifr)
+			continue;
+
+		memset(ebuf, 0, sizeof(ebuf));
+		strlcpy(entry->intf_name, lifr->lifr_name,
+		    sizeof(entry->intf_name));
+		entry->intf_len = sizeof(ebuf);
+
+		/* Repair the alias name back up */
+		if (p) *p = ':';
+
+		/* Ignore IPMP interfaces. These are virtual interfaces made up
+		 * of physical interfaces. IPMP interfaces do not support things
+		 * like packet sniffing; it is necessary to use one of the
+		 * underlying physical interfaces instead. This works as long as
+		 * the physical interface's test address is on the same subnet
+		 * as the IPMP interface's address. */
+		if (ioctl(intf->fd, SIOCGLIFFLAGS, lifr) >= 0)
+			;
+		else if (intf->fd6 != -1 && ioctl(intf->fd6, SIOCGLIFFLAGS, lifr) >= 0)
+			;
+		else
+			return (-1);
+#ifdef IFF_IPMP
+		if (lifr->lifr_flags & IFF_IPMP) {
+			continue;
+		}
+#endif
+		
+		if (_intf_get_noalias(intf, entry) < 0)
+			return (-1);
+		if (_intf_get_aliases(intf, entry) < 0)
+			return (-1);
+		
+		if ((ret = (*callback)(entry, arg)) != 0)
+			return (ret);
+	}
+	return (0);
+}
 #else
 int
 intf_loop(intf_t *intf, intf_handler callback, void *arg)
@@ -712,7 +997,7 @@ intf_loop(intf_t *intf, intf_handler callback, void *arg)
 			*p = '\0';
 		
 		if (pifr != NULL && strcmp(ifr->ifr_name, pifr->ifr_name) == 0) {
-		  if (p) *p = ':';
+			if (p) *p = ':';
 			continue;
 		}
 
@@ -720,9 +1005,10 @@ intf_loop(intf_t *intf, intf_handler callback, void *arg)
 		strlcpy(entry->intf_name, ifr->ifr_name,
 		    sizeof(entry->intf_name));
 		entry->intf_len = sizeof(ebuf);
-		
-		/* Repair the alias name back up. */
+
+		/* Repair the alias name back up */
 		if (p) *p = ':';
+		
 		if (_intf_get_noalias(intf, entry) < 0)
 			return (-1);
 		if (_intf_get_aliases(intf, entry) < 0)
