@@ -4,7 +4,7 @@
  * Copyright (c) 2001 Dug Song <dugsong@monkey.org>
  * Copyright (c) 1999 Masaki Hirabaru <masaki@merit.edu>
  * 
- * $Id: route-bsd.c,v 1.22 2005/02/10 05:18:38 dugsong Exp $
+ * $Id: route-bsd.c 555 2005-02-10 05:18:38Z dugsong $
  */
 
 #include "config.h"
@@ -28,10 +28,14 @@
 #include <sys/stream.h>
 #include <sys/stropts.h>
 #endif
+#ifdef HAVE_GETKERNINFO
+#include <sys/kinfo.h>
+#endif
 
 #define route_t	oroute_t	/* XXX - unixware */
 #include <net/route.h>
 #undef route_t
+#include <net/if.h>
 #include <netinet/in.h>
 
 #include <errno.h>
@@ -44,7 +48,7 @@
 #include "dnet.h"
 
 #define ROUNDUP(a) \
-	((a) > 0 ? (1 + (((a) - 1) | (sizeof(long) - 1))) : sizeof(long))
+	((a) > 0 ? (1 + (((a) - 1) | (RT_MSGHDR_ALIGNMENT - 1))) : RT_MSGHDR_ALIGNMENT)
 
 #ifdef HAVE_SOCKADDR_SA_LEN
 #define NEXTSA(s) \
@@ -73,7 +77,7 @@ route_msg_print(struct rt_msghdr *rtm)
 #endif
 
 static int
-route_msg(route_t *r, int type, struct addr *dst, struct addr *gw)
+route_msg(route_t *r, int type, char intf_name[INTF_NAME_LEN], struct addr *dst, struct addr *gw)
 {
 	struct addr net;
 	struct rt_msghdr *rtm;
@@ -150,6 +154,16 @@ route_msg(route_t *r, int type, struct addr *dst, struct addr *gw)
 			errno = ESRCH;
 			return (-1);
 		}
+
+		if (intf_name != NULL) {
+			char namebuf[IF_NAMESIZE];
+
+			if (if_indextoname(rtm->rtm_index, namebuf) == NULL) {
+				errno = ESRCH;
+				return (-1);
+			}
+			strlcpy(intf_name, namebuf, sizeof(intf_name));
+		}
 	}
 	return (0);
 }
@@ -182,7 +196,7 @@ route_add(route_t *r, const struct route_entry *entry)
 	
 	memcpy(&rtent, entry, sizeof(rtent));
 	
-	if (route_msg(r, RTM_ADD, &rtent.route_dst, &rtent.route_gw) < 0)
+	if (route_msg(r, RTM_ADD, NULL, &rtent.route_dst, &rtent.route_gw) < 0)
 		return (-1);
 	
 	return (0);
@@ -198,7 +212,7 @@ route_delete(route_t *r, const struct route_entry *entry)
 	if (route_get(r, &rtent) < 0)
 		return (-1);
 	
-	if (route_msg(r, RTM_DELETE, &rtent.route_dst, &rtent.route_gw) < 0)
+	if (route_msg(r, RTM_DELETE, NULL, &rtent.route_dst, &rtent.route_gw) < 0)
 		return (-1);
 	
 	return (0);
@@ -207,13 +221,41 @@ route_delete(route_t *r, const struct route_entry *entry)
 int
 route_get(route_t *r, struct route_entry *entry)
 {
-	if (route_msg(r, RTM_GET, &entry->route_dst, &entry->route_gw) < 0)
+	if (route_msg(r, RTM_GET, entry->intf_name, &entry->route_dst, &entry->route_gw) < 0)
 		return (-1);
+	entry->intf_name[0] = '\0';
 	
 	return (0);
 }
 
-#if defined(HAVE_SYS_SYSCTL_H) || defined(HAVE_STREAMS_ROUTE)
+#if defined(HAVE_SYS_SYSCTL_H) || defined(HAVE_STREAMS_ROUTE) || defined(HAVE_GETKERNINFO)
+/* This wrapper around addr_ston, on failure, checks for a gateway address
+ * family of AF_LINK, and if it finds one, stores an all-zero address of the
+ * same type as dst. The all-zero address is a convention for same-subnet
+ * routing table entries. */
+static int
+addr_ston_gateway(const struct addr *dst,
+	const struct sockaddr *sa, struct addr *a)
+{
+	int rc;
+
+	rc = addr_ston(sa, a);
+	if (rc == 0)
+		return rc;
+
+#ifdef HAVE_NET_IF_DL_H
+# ifdef AF_LINK
+	if (sa->sa_family == AF_LINK) {
+		memset(a, 0, sizeof(*a));
+		a->addr_type = dst->addr_type;
+		return (0);
+	}
+# endif
+#endif
+
+	return (-1);
+}
+
 int
 route_loop(route_t *r, route_handler callback, void *arg)
 {
@@ -241,6 +283,21 @@ route_loop(route_t *r, route_handler callback, void *arg)
 	}
 	lim = buf + len;
 	next = buf;
+#elif defined(HAVE_GETKERNINFO)
+	int len = getkerninfo(KINFO_RT_DUMP,0,0,0);
+
+	if (len == 0)
+		return (0);
+
+	if ((buf = malloc(len)) == NULL)
+		return (-1);
+
+	if (getkerninfo(KINFO_RT_DUMP,buf,&len,0) < 0) {
+		free(buf);
+		return (-1);
+	}
+	lim = buf + len;
+	next = buf;
 #else /* HAVE_STREAMS_ROUTE */
 	struct rt_giarg giarg, *gp;
 
@@ -266,17 +323,29 @@ route_loop(route_t *r, route_handler callback, void *arg)
 	lim = buf + gp->gi_size;
 	next = buf + sizeof(giarg);
 #endif
+	/* This loop assumes that RTA_DST, RTA_GATEWAY, and RTA_NETMASK have the
+	 * values, 1, 2, and 4 respectively. Cf. Unix Network Programming,
+	 * p. 494, function get_rtaddrs. */
 	for (ret = 0; next < lim; next += rtm->rtm_msglen) {
+		char namebuf[IF_NAMESIZE];
 		rtm = (struct rt_msghdr *)next;
 		sa = (struct sockaddr *)(rtm + 1);
 
-		if (addr_ston(sa, &entry.route_dst) < 0 ||
-		    (rtm->rtm_addrs & RTA_GATEWAY) == 0)
+		if (if_indextoname(rtm->rtm_index, namebuf) == NULL)
+			continue;
+		strlcpy(entry.intf_name, namebuf, sizeof(entry.intf_name));
+
+		if ((rtm->rtm_addrs & RTA_DST) == 0)
+			/* Need a destination. */
+			continue;
+		if (addr_ston(sa, &entry.route_dst) < 0)
 			continue;
 
+		if ((rtm->rtm_addrs & RTA_GATEWAY) == 0)
+			/* Need a gateway. */
+			continue;
 		sa = NEXTSA(sa);
-		
-		if (addr_ston(sa, &entry.route_gw) < 0)
+		if (addr_ston_gateway(&entry.route_dst, sa, &entry.route_gw) < 0)
 			continue;
 		
 		if (entry.route_dst.addr_type != entry.route_gw.addr_type ||
@@ -289,6 +358,7 @@ route_loop(route_t *r, route_handler callback, void *arg)
 			if (addr_stob(sa, &entry.route_dst.addr_bits) < 0)
 				continue;
 		}
+
 		if ((ret = callback(&entry, arg)) != 0)
 			break;
 	}
@@ -308,13 +378,11 @@ int
 route_loop(route_t *r, route_handler callback, void *arg)
 {
 	struct route_entry entry;
-	struct sockaddr_in sin;
 	struct strbuf msg;
 	struct T_optmgmt_req *tor;
 	struct T_optmgmt_ack *toa;
 	struct T_error_ack *tea;
 	struct opthdr *opt;
-	mib2_ipRouteEntry_t *rt, *rtend;
 	u_char buf[8192];
 	int flags, rc, rtable, ret;
 
@@ -342,6 +410,8 @@ route_loop(route_t *r, route_handler callback, void *arg)
 	msg.maxlen = sizeof(buf);
 	
 	for (;;) {
+		mib2_ipRouteEntry_t *rt, *rtend;
+
 		flags = 0;
 		if ((rc = getmsg(r->ip_fd, &msg, NULL, &flags)) < 0)
 			return (-1);
@@ -368,6 +438,8 @@ route_loop(route_t *r, route_handler callback, void *arg)
 		flags = 0;
 		
 		do {
+			struct sockaddr_in sin;
+
 			rc = getmsg(r->ip_fd, NULL, &msg, &flags);
 			
 			if (rc != 0 && rc != MOREDATA)
@@ -388,6 +460,8 @@ route_loop(route_t *r, route_handler callback, void *arg)
 				    rt->ipRouteNextHop == IP_ADDR_ANY)
 					continue;
 				
+				entry.intf_name[0] = '\0';
+
 				sin.sin_addr.s_addr = rt->ipRouteNextHop;
 				addr_ston((struct sockaddr *)&sin,
 				    &entry.route_gw);
@@ -405,6 +479,98 @@ route_loop(route_t *r, route_handler callback, void *arg)
 			}
 		} while (rc == MOREDATA);
 	}
+
+	tor = (struct T_optmgmt_req *)buf;
+	toa = (struct T_optmgmt_ack *)buf;
+	tea = (struct T_error_ack *)buf;
+
+	tor->PRIM_type = T_OPTMGMT_REQ;
+	tor->OPT_offset = sizeof(*tor);
+	tor->OPT_length = sizeof(*opt);
+	tor->MGMT_flags = T_CURRENT;
+	
+	opt = (struct opthdr *)(tor + 1);
+	opt->level = MIB2_IP6;
+	opt->name = opt->len = 0;
+	
+	msg.maxlen = sizeof(buf);
+	msg.len = sizeof(*tor) + sizeof(*opt);
+	msg.buf = buf;
+	
+	if (putmsg(r->ip_fd, &msg, NULL, 0) < 0)
+		return (-1);
+	
+	opt = (struct opthdr *)(toa + 1);
+	msg.maxlen = sizeof(buf);
+	
+	for (;;) {
+		mib2_ipv6RouteEntry_t *rt, *rtend;
+
+		flags = 0;
+		if ((rc = getmsg(r->ip_fd, &msg, NULL, &flags)) < 0)
+			return (-1);
+
+		/* See if we're finished. */
+		if (rc == 0 &&
+		    msg.len >= sizeof(*toa) &&
+		    toa->PRIM_type == T_OPTMGMT_ACK &&
+		    toa->MGMT_flags == T_SUCCESS && opt->len == 0)
+			break;
+
+		if (msg.len >= sizeof(*tea) && tea->PRIM_type == T_ERROR_ACK)
+			return (-1);
+		
+		if (rc != MOREDATA || msg.len < (int)sizeof(*toa) ||
+		    toa->PRIM_type != T_OPTMGMT_ACK ||
+		    toa->MGMT_flags != T_SUCCESS)
+			return (-1);
+		
+		rtable = (opt->level == MIB2_IP6 && opt->name == MIB2_IP6_ROUTE);
+		
+		msg.maxlen = sizeof(buf) - (sizeof(buf) % sizeof(*rt));
+		msg.len = 0;
+		flags = 0;
+		
+		do {
+			struct sockaddr_in6 sin6;
+
+			rc = getmsg(r->ip_fd, NULL, &msg, &flags);
+			
+			if (rc != 0 && rc != MOREDATA)
+				return (-1);
+			
+			if (!rtable)
+				continue;
+			
+			rt = (mib2_ipv6RouteEntry_t *)msg.buf;
+			rtend = (mib2_ipv6RouteEntry_t *)(msg.buf + msg.len);
+
+			sin6.sin6_family = AF_INET6;
+
+			for ( ; rt < rtend; rt++) {
+				if ((rt->ipv6RouteInfo.re_ire_type &
+				    (IRE_BROADCAST|IRE_ROUTE_REDIRECT|
+					IRE_LOCAL|IRE_ROUTE)) != 0 ||
+				    memcmp(&rt->ipv6RouteNextHop, IP6_ADDR_UNSPEC, IP6_ADDR_LEN) == 0)
+					continue;
+				
+				entry.intf_name[0] = '\0';
+
+				sin6.sin6_addr = rt->ipv6RouteNextHop;
+				addr_ston((struct sockaddr *)&sin6,
+				    &entry.route_gw);
+				
+				sin6.sin6_addr = rt->ipv6RouteDest;
+				addr_ston((struct sockaddr *)&sin6,
+				    &entry.route_dst);
+				
+				entry.route_dst.addr_bits = rt->ipv6RoutePfxLength;
+				
+				if ((ret = callback(&entry, arg)) != 0)
+					return (ret);
+			}
+		} while (rc == MOREDATA);
+	}
 	return (0);
 }
 #elif defined(HAVE_NET_RADIX_H)
@@ -431,6 +597,7 @@ _radix_walk(int fd, struct radix_node *rn, route_handler callback, void *arg)
 	_kread(fd, rn, &rnode, sizeof(rnode));
 	if (rnode.rn_b < 0) {
 		if (!(rnode.rn_flags & RNF_ROOT)) {
+			entry.intf_name[0] = '\0';
 			_kread(fd, rn, &rt, sizeof(rt));
 			_kread(fd, rt_key(&rt), &sin, sizeof(sin));
 			addr_ston((struct sockaddr *)&sin, &entry.route_dst);
