@@ -9,7 +9,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -19,11 +19,12 @@
  * technology into proprietary software, we sell alternative licenses      *
  * (contact sales@insecure.com).  Dozens of software vendors already       *
  * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * detection, version detection, and the Nmap Scripting Engine.            *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
  * "derivative work" for the purpose of this license if it does any of the *
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
@@ -31,40 +32,32 @@
  *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                * 
+ *   not derivative works.)                                                *
  * o Integrates/includes/aggregates Nmap into a proprietary executable     *
  *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is just meant to        *
- * clarify our interpretation of derived works with some common examples.  *
- * These restrictions only apply when you actually redistribute Nmap.  For *
- * example, nothing stops you from writing and selling a proprietary       *
- * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://nmap.org to download Nmap.                                       *
- *                                                                         *
- * We don't consider these to be added restrictions on top of the GPL, but *
- * just a clarification of how we interpret "derived works" as it applies  *
- * to our GPL-licensed Nmap product.  This is similar to the way Linus     *
- * Torvalds has announced his interpretation of how "derived works"        *
- * applies to Linux kernel modules.  Our interpretation refers only to     *
- * Nmap - we don't speak for any other GPL products.                       *
+ * works of Nmap, as well as other software we distribute under this       *
+ * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+ * but is meant to clarify our interpretation of derived works with some   *
+ * common examples.  Our interpretation applies only to Nmap--we don't     *
+ * speak for other people's GPL works.                                     *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included COPYING.OpenSSL file, and distribute linked      *
- * combinations including the two. You must obey the GNU GPL in all        *
+ * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+ * linked combinations including the two. You must obey the GNU GPL in all *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
  * but you are not obligated to do so.                                     *
@@ -80,17 +73,18 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to fyodor@insecure.org for possible incorporation into the main         *
+ * to nmap-dev@insecure.org for possible incorporation into the main       *
  * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
- * to reuse, modify, and relicense the code.  Nmap will always be          *
- * available Open Source, but this is important because the inability to   *
- * relicense code has caused devastating problems for other Free Software  *
- * projects (such as KDE and NASM).  We also occasionally relicense the    *
- * code to third parties as discussed above.  If you wish to specify       *
- * special license conditions of your contributions, just say so when you  *
- * send them.                                                              *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -101,7 +95,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nbase.h 9839 2008-08-29 20:56:53Z david $ */
+/* $Id: nbase.h 30067 2012-10-20 15:00:10Z henri $ */
 
 #ifndef NBASE_H
 #define NBASE_H
@@ -155,6 +149,11 @@
 
 #include <stdlib.h>
 #include <ctype.h>
+
+#if HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
+
 #if HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
@@ -171,6 +170,12 @@
 #include <netdb.h>  
 #endif
 
+#if HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+
+#include <stdio.h>
+
 #ifndef MAXHOSTNAMELEN
 #define MAXHOSTNAMELEN 64
 #endif
@@ -206,9 +211,31 @@ typedef int64_t s64;
 #define MIN(x,y) (((x)<(y))?(x):(y))
 #endif
 #ifndef ABS
-#define ABS(x) (((x) >= 0)?(x):(-x)) 
+#define ABS(x) (((x) >= 0)?(x):-(x)) 
 #endif
 
+/* Timeval subtraction in microseconds */
+#define TIMEVAL_SUBTRACT(a,b) (((a).tv_sec - (b).tv_sec) * 1000000 + (a).tv_usec - (b).tv_usec)
+/* Timeval subtract in milliseconds */
+#define TIMEVAL_MSEC_SUBTRACT(a,b) ((((a).tv_sec - (b).tv_sec) * 1000) + ((a).tv_usec - (b).tv_usec) / 1000)
+/* Timeval subtract in seconds; truncate towards zero */
+#define TIMEVAL_SEC_SUBTRACT(a,b) ((a).tv_sec - (b).tv_sec + (((a).tv_usec < (b).tv_usec) ? - 1 : 0))
+/* Timeval subtract in fractional seconds; convert to float */
+#define TIMEVAL_FSEC_SUBTRACT(a,b) ((a).tv_sec - (b).tv_sec + (((a).tv_usec - (b).tv_usec)/1000000.0))
+
+/* assign one timeval to another timeval plus some msecs: a = b + msecs */
+#define TIMEVAL_MSEC_ADD(a, b, msecs) { (a).tv_sec = (b).tv_sec + ((msecs) / 1000); (a).tv_usec = (b).tv_usec + ((msecs) % 1000) * 1000; (a).tv_sec += (a).tv_usec / 1000000; (a).tv_usec %= 1000000; }
+#define TIMEVAL_ADD(a, b, usecs) { (a).tv_sec = (b).tv_sec + ((usecs) / 1000000); (a).tv_usec = (b).tv_usec + ((usecs) % 1000000); (a).tv_sec += (a).tv_usec / 1000000; (a).tv_usec %= 1000000; }
+
+/* Find our if one timeval is before or after another, avoiding the integer
+   overflow that can result when doing a TIMEVAL_SUBTRACT on two widely spaced
+   timevals. */
+#define TIMEVAL_BEFORE(a, b) (((a).tv_sec < (b).tv_sec) || ((a).tv_sec == (b).tv_sec && (a).tv_usec < (b).tv_usec))
+#define TIMEVAL_AFTER(a, b) (((a).tv_sec > (b).tv_sec) || ((a).tv_sec == (b).tv_sec && (a).tv_usec > (b).tv_usec))
+
+/* Convert a timeval to floating point seconds */
+#define TIMEVAL_SECS(a) ((double) (a).tv_sec + (double) (a).tv_usec / 1000000)
+
 
 /* sprintf family */
 #if !defined(HAVE_SNPRINTF) && defined(__cplusplus)
@@ -280,8 +307,9 @@ extern "C" int vsnprintf (char *, size_t, const char *, va_list);
 #define R_OK 04
 #endif
 
+/* wtf was ms thinking? */
 #define access _access
-#define stat _stat /* wtf was ms thinking? */
+#define stat _stat
 #define execve _execve
 #define getpid _getpid
 #define dup _dup
@@ -325,6 +353,13 @@ extern "C" {
      or EINTR. */
 int socket_errno();
 
+/* We can't just use strerror to get socket errors on Windows because it has
+   its own set of error codes: WSACONNRESET not ECONNRESET for example. This
+   function will do the right thing on Windows. Call it like
+     socket_strerror(socket_errno())
+*/
+char *socket_strerror(int errnum);
+
 /* The usleep() function is important as well */
 #ifndef HAVE_USLEEP
 #if defined( HAVE_NANOSLEEP) || defined(WIN32)
@@ -365,13 +400,35 @@ int Vsnprintf(char *, size_t, const char *, va_list)
 int Snprintf(char *, size_t, const char *, ...)
      __attribute__ ((format (printf, 3, 4)));
 
+int alloc_vsprintf(char **strp, const char *fmt, va_list va)
+     __attribute__ ((format (printf, 2, 0)));
+
 /* Trivial function that returns nonzero if all characters in str of
    length strlength are printable (as defined by isprint()) */
 int stringisprintable(const char *str, int strlength);
 
+/* parse_long is like strtol or atoi, but it allows digits only.
+   No whitespace, sign, or radix prefix. */
+long parse_long(const char *s, char **tail);
+
+/* This function takes a byte count and stores a short ascii equivalent
+   in the supplied buffer. Eg: 0.122MB, 10.322Kb or 128B. */
+char *format_bytecount(unsigned long long bytes, char *buf, size_t buflen);
+
+/* Compare a canonical option name (e.g. "max-scan-delay") with a
+   user-generated option such as "max_scan_delay" and returns 0 if the
+   two values are considered equivalant (for example, - and _ are
+   considered to be the same), nonzero otherwise. */
+int optcmp(const char *a, const char *b);
+
 /* Convert non-printable characters to replchar in the string */
 void replacenonprintable(char *str, int strlength, char replchar);
 
+/* Returns one if the file pathname given exists, is not a directory and
+ * is readable by the executing process.  Returns two if it is readable
+ * and is a directory.  Otherwise returns 0. */
+int file_is_readable(const char *pathname);
+
 /* Portable, incompatible replacements for dirname and basename. */
 char *path_get_dirname(const char *path);
 char *path_get_basename(const char *path);
@@ -392,14 +449,52 @@ unsigned int get_random_uint();
 u32 get_random_u32();
 u16 get_random_u16();
 u8 get_random_u8();
+u32 get_random_unique_u32();
 
+/* Create a new socket inheritable by subprocesses. On non-Windows systems it's
+   just a normal socket. */
+int inheritable_socket(int af, int style, int protocol);
+/* The dup function on Windows works only on file descriptors, not socket
+   handles. This function accomplishes the same thing for sockets. */
+int dup_socket(int sd);
 int unblock_socket(int sd);
 int block_socket(int sd);
+int socket_bindtodevice(int sd, const char *device);
 
-/* CRC16 Cyclic Redundancy Check  */
-unsigned long crc16(unsigned char *buf, int len);
+/* CRC32 Cyclic Redundancy Check */
+unsigned long nbase_crc32(unsigned char *buf, int len);
+/* CRC32C Cyclic Redundancy Check (Castagnoli) */
+unsigned long nbase_crc32c(unsigned char *buf, int len);
+/* Adler32 Checksum */
+unsigned long nbase_adler32(unsigned char *buf, int len);
 
-long tval2msecs(char *tspec);
+double tval2secs(const char *tspec);
+long tval2msecs(const char *tspec);
+const char *tval_unit(const char *tspec);
+
+int fselect(int s, fd_set *rmaster, fd_set *wmaster, fd_set *emaster, struct timeval *tv);
+
+char *hexdump(const u8 *cp, u32 length);
+
+char *executable_path(const char *argv0);
+
+/* addrset management functions and definitions */
+/* A set of addresses. Used to match against allow/deny lists. */
+struct addrset_elem;
+
+/* A set of addresses. Used to match against allow/deny lists. */
+struct addrset {
+    /* Linked list of struct addset_elem. */
+    struct addrset_elem *head;
+};
+
+void nbase_set_log(void (*log_user_func)(const char *, ...),void (*log_debug_func)(const char *, ...));
+extern void addrset_init(struct addrset *set);
+extern void addrset_free(struct addrset *set);
+extern void addrset_elem_print(FILE *fp, const struct addrset_elem *elem);
+extern int addrset_add_spec(struct addrset *set, const char *spec, int af, int dns);
+extern int addrset_add_file(struct addrset *set, FILE *fd, int af, int dns);
+extern int addrset_contains(const struct addrset *set, const struct sockaddr *sa);
 
 #ifndef STDIN_FILENO
 #define STDIN_FILENO 0
