@@ -11,7 +11,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -21,11 +21,12 @@
  * technology into proprietary software, we sell alternative licenses      *
  * (contact sales@insecure.com).  Dozens of software vendors already       *
  * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * detection, version detection, and the Nmap Scripting Engine.            *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
  * "derivative work" for the purpose of this license if it does any of the *
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
@@ -33,40 +34,32 @@
  *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                * 
+ *   not derivative works.)                                                *
  * o Integrates/includes/aggregates Nmap into a proprietary executable     *
  *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is just meant to        *
- * clarify our interpretation of derived works with some common examples.  *
- * These restrictions only apply when you actually redistribute Nmap.  For *
- * example, nothing stops you from writing and selling a proprietary       *
- * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://nmap.org to download Nmap.                                       *
- *                                                                         *
- * We don't consider these to be added restrictions on top of the GPL, but *
- * just a clarification of how we interpret "derived works" as it applies  *
- * to our GPL-licensed Nmap product.  This is similar to the way Linus     *
- * Torvalds has announced his interpretation of how "derived works"        *
- * applies to Linux kernel modules.  Our interpretation refers only to     *
- * Nmap - we don't speak for any other GPL products.                       *
+ * works of Nmap, as well as other software we distribute under this       *
+ * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+ * but is meant to clarify our interpretation of derived works with some   *
+ * common examples.  Our interpretation applies only to Nmap--we don't     *
+ * speak for other people's GPL works.                                     *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included COPYING.OpenSSL file, and distribute linked      *
- * combinations including the two. You must obey the GNU GPL in all        *
+ * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+ * linked combinations including the two. You must obey the GNU GPL in all *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
  * but you are not obligated to do so.                                     *
@@ -82,17 +75,18 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to fyodor@insecure.org for possible incorporation into the main         *
+ * to nmap-dev@insecure.org for possible incorporation into the main       *
  * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
- * to reuse, modify, and relicense the code.  Nmap will always be          *
- * available Open Source, but this is important because the inability to   *
- * relicense code has caused devastating problems for other Free Software  *
- * projects (such as KDE and NASM).  We also occasionally relicense the    *
- * code to third parties as discussed above.  If you wish to specify       *
- * special license conditions of your contributions, just say so when you  *
- * send them.                                                              *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -103,9 +97,10 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nbase_rnd.c 7641 2008-05-22 20:45:49Z fyodor $ */
+/* $Id: nbase_rnd.c 28551 2012-05-11 15:38:05Z david $ */
 
 #include "nbase.h"
+#include <errno.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -192,8 +187,12 @@ void nrand_init(nrand_h *r) {
   /* Try to fill the rest of the state with OS provided entropy */
   if ((fd = open("/dev/urandom", O_RDONLY)) != -1 ||
       (fd = open("/dev/arandom", O_RDONLY)) != -1) {
-    read(fd, seed + sizeof(*tv) + sizeof(*pid),
-	 sizeof(seed) - sizeof(*tv) - sizeof(*pid));
+    ssize_t n;
+    do {
+      errno = 0;
+      n = read(fd, seed + sizeof(*tv) + sizeof(*pid),
+               sizeof(seed) - sizeof(*tv) - sizeof(*pid));
+    } while (n < 0 && errno == EINTR);
     close(fd);
   }
 #endif
@@ -263,3 +262,125 @@ unsigned short get_random_ushort() {
   get_random_bytes(&s, sizeof(unsigned short));
   return s;
 }
+
+
+/* This function is magic ;-)
+ * 
+ * Sometimes Nmap wants to generate IPs that look random
+ * but don't have any duplicates.  The strong RC4 generator
+ * can't be used for this purpose because it can generate duplicates
+ * if you get enough IPs (birthday paradox).
+ *
+ * This routine exploits the fact that a LCG won't repeat for the
+ * entire duration of its period.  An LCG has some pretty bad
+ * properties though so this routine does extra work to try to
+ * tweak the LCG output so that is has very good statistics but
+ * doesn't repeat.  The tweak used was mostly made up on the spot
+ * but is generally based on good ideas and has been moderately
+ * tested.  See links and reasoning below.
+ */
+u32 get_random_unique_u32() {
+  static u32 state, tweak1, tweak2, tweak3;
+  static int state_init = 0;
+  u32 output;
+  
+  /* Initialize if we need to */
+  if (!state_init) {
+    get_random_bytes(&state, sizeof(state));
+    get_random_bytes(&tweak1, sizeof(tweak1));
+    get_random_bytes(&tweak2, sizeof(tweak2));
+    get_random_bytes(&tweak3, sizeof(tweak3));
+
+    state_init = 1;
+  }
+
+  /* What is this math crap?
+   *
+   * The whole idea behind this generator is that an LCG can be constructed
+   * with a period of exactly 2^32.  As long as the LCG is fed back onto
+   * itself the period will be 2^32.  The tweak after the LCG is just
+   * a good permutation in GF(2^32).
+   *
+   * To accomplish the tweak the notion of rounds and round keys from
+   * block ciphers has been borrowed.  The only special aspect of this
+   * block cipher is that the first round short-circuits the LCG.
+   *
+   * This block cipher uses three rounds.  Each round is as follows:
+   *
+   * 1) Affine transform in GF(2^32)
+   * 2) Rotate left by round constant
+   * 3) XOR with round key
+   *
+   * For round one the affine transform is used as an LCG.
+   */
+
+  /* Reasoning:
+   *
+   * Affine transforms were chosen both to make a LCG and also
+   * to try to introduce non-linearity.
+   *
+   * The rotate up each round was borrowed from SHA-1 and was introduced
+   * to help obscure the obvious short cycles when you truncate an LCG with
+   * a power-of-two period like the one used.
+   *
+   * The XOR with the round key was borrowed from several different
+   * published functions (but see Xorshift)
+   * and provides a different sequence for the full LCG.
+   * There are 3 32 bit round keys.  This generator can
+   * generate 2^96 different sequences of period 2^32.
+   *
+   * This generator was tested with Dieharder.  It did not fail any test.
+   */
+
+  /* See:
+   *
+   * http://en.wikipedia.org/wiki/Galois_field
+   * http://en.wikipedia.org/wiki/Affine_cipher
+   * http://en.wikipedia.org/wiki/Linear_congruential_generator
+   * http://en.wikipedia.org/wiki/Xorshift
+   * http://en.wikipedia.org/wiki/Sha-1
+   *
+   * http://seclists.org/nmap-dev/2009/q3/0695.html
+   */
+
+
+  /* First off, we need to evolve the state with our LCG
+   * We'll use the LCG from Numerical Recipes (m=2^32,
+   * a=1664525, c=1013904223).  All by itself this generator
+   * pretty bad.  We're going to try to fix that without causing
+   * duplicates.
+   */
+  state = (((state * 1664525) & 0xFFFFFFFF) + 1013904223) & 0xFFFFFFFF;
+ 
+  output = state;
+
+  /* With a normal LCG, we would just output the state.
+   * In this case, though, we are going to try to destroy the
+   * linear correlation between IPs by approximating a random permutation
+   * in GF(2^32) (collision-free)
+   */
+
+  /* Then rotate and XOR */
+  output = ((output << 7) | (output >> (32 - 7)));
+  output = output ^ tweak1; /* This is the round key */
+
+  /* End round 1, start round 2 */
+
+  /* Then put it through an affine transform (glibc constants) */
+  output = (((output * 1103515245) & 0xFFFFFFFF) + 12345) & 0xFFFFFFFF;
+
+  /* Then rotate and XOR some more */
+  output = ((output << 15) | (output >> (32 - 15)));
+  output = output ^ tweak2;
+
+  /* End round 2, start round 3 */
+
+  /* Then put it through another affine transform (Quick C/C++ constants) */
+  output = (((output * 214013) & 0xFFFFFFFF) + 2531011) & 0xFFFFFFFF;
+
+  /* Then rotate and XOR some more */
+  output = ((output << 5) | (output >> (32 - 5)));
+  output = output ^ tweak3;
+
+  return output;
+}
