@@ -4,7 +4,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2011 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -14,11 +14,12 @@
  * technology into proprietary software, we sell alternative licenses      *
  * (contact sales@insecure.com).  Dozens of software vendors already       *
  * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * detection, version detection, and the Nmap Scripting Engine.            *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
  * "derivative work" for the purpose of this license if it does any of the *
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
@@ -32,19 +33,20 @@
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is meant to clarify our *
- * interpretation of derived works with some common examples.  Our         *
- * interpretation applies only to Nmap--we don't speak for other people's  *
- * GPL works.                                                              *
+ * works of Nmap, as well as other software we distribute under this       *
+ * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+ * but is meant to clarify our interpretation of derived works with some   *
+ * common examples.  Our interpretation applies only to Nmap--we don't     *
+ * speak for other people's GPL works.                                     *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
@@ -68,15 +70,16 @@
  * and add new features.  You are highly encouraged to send your changes   *
  * to nmap-dev@insecure.org for possible incorporation into the main       *
  * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering the Nmap Project (Insecure.Com LLC) the unlimited,             *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because the *
- * inability to relicense code has caused devastating problems for other   *
- * Free Software projects (such as KDE and NASM).  We also occasionally    *
- * relicense the code to third parties as discussed above.  If you wish to *
- * specify special license conditions of your contributions, just say so   *
- * when you send them.                                                     *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -87,7 +90,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap_error.cc 21904 2011-01-21 00:04:16Z fyodor $ */
+/* $Id: nmap_error.cc 29083 2012-06-30 01:51:30Z david $ */
 
 #include "nmap_error.h"
 #include "output.h"
@@ -110,15 +113,12 @@ void fatal(const char *fmt, ...) {
   timep = time(NULL);
 
   va_start(ap, fmt);
-  log_vwrite(LOG_STDERR, fmt, ap);
+  log_vwrite(LOG_NORMAL|LOG_STDERR, fmt, ap);
   va_end(ap);
-  if (o.log_errors) {
-    va_start(ap, fmt);
-    log_vwrite(LOG_NORMAL, fmt, ap);
-    va_end(ap);
-  }
-  log_write(o.log_errors? LOG_NORMAL|LOG_STDERR : LOG_STDERR, "\nQUITTING!\n");
+  log_write(LOG_NORMAL|LOG_STDERR, "\nQUITTING!\n");
 
+  if (xml_tag_open())
+    xml_close_start_tag();
   if (!xml_root_written())
     xml_start_tag("nmaprun");
   /* Close all open XML elements but one. */
@@ -156,15 +156,9 @@ void error(const char *fmt, ...) {
   va_list  ap;
 
   va_start(ap, fmt);
-  log_vwrite(LOG_STDERR, fmt, ap);
+  log_vwrite(LOG_NORMAL|LOG_STDERR, fmt, ap);
   va_end(ap);
-  
-  if (o.log_errors) {
-    va_start(ap, fmt);
-    log_vwrite(LOG_NORMAL, fmt, ap);
-    va_end(ap);
-  }
-  log_write(o.log_errors? LOG_NORMAL|LOG_STDERR : LOG_STDERR, "\n");
+  log_write(LOG_NORMAL|LOG_STDERR , "\n");
   return;
 }
 
@@ -192,9 +186,11 @@ void pfatal(const char *fmt, ...) {
   Vsnprintf(errbuf, sizeof(errbuf), fmt, ap);
   va_end(ap);
 
-  log_write(o.log_errors? LOG_NORMAL|LOG_STDERR : LOG_STDERR, "%s: %s (%d)\n", 
+  log_write(LOG_NORMAL|LOG_STDERR, "%s: %s (%d)\n",
 	    errbuf, strerror_s, error_number);
 
+  if (xml_tag_open())
+    xml_close_start_tag();
   if (!xml_root_written())
     xml_start_tag("nmaprun");
   /* Close all open XML elements but one. */
@@ -223,7 +219,7 @@ void pfatal(const char *fmt, ...) {
   HeapFree(GetProcessHeap(), 0, strerror_s);
 #endif
 
-  if (o.log_errors) log_flush(LOG_NORMAL);
+  log_flush(LOG_NORMAL);
   fflush(stderr);
   exit(1);
 }
@@ -246,21 +242,16 @@ void gh_perror(const char *fmt, ...) {
 #endif
   
   va_start(ap, fmt);
-  log_vwrite(LOG_STDERR, fmt, ap);
+  log_vwrite(LOG_NORMAL|LOG_STDERR, fmt, ap);
   va_end(ap);
-  if (o.log_errors) {
-      va_start(ap, fmt);
-      log_vwrite(LOG_NORMAL, fmt, ap);
-      va_end(ap);
-  }
-  log_write(o.log_errors? LOG_NORMAL|LOG_STDERR : LOG_STDERR, ": %s (%d)\n",
+  log_write(LOG_NORMAL|LOG_STDERR, ": %s (%d)\n",
     strerror_s, error_number);
 
 #ifdef WIN32
   HeapFree(GetProcessHeap(), 0, strerror_s);
 #endif
 
-  if (o.log_errors) log_flush(LOG_NORMAL);
+  log_flush(LOG_NORMAL);
   fflush(stderr);
   return;
 }
