@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -15,11 +15,12 @@
  * technology into proprietary software, we sell alternative licenses      *
  * (contact sales@insecure.com).  Dozens of software vendors already       *
  * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * detection, version detection, and the Nmap Scripting Engine.            *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
  * "derivative work" for the purpose of this license if it does any of the *
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
@@ -27,40 +28,32 @@
  *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                * 
+ *   not derivative works.)                                                *
  * o Integrates/includes/aggregates Nmap into a proprietary executable     *
  *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is just meant to        *
- * clarify our interpretation of derived works with some common examples.  *
- * These restrictions only apply when you actually redistribute Nmap.  For *
- * example, nothing stops you from writing and selling a proprietary       *
- * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://nmap.org to download Nmap.                                       *
- *                                                                         *
- * We don't consider these to be added restrictions on top of the GPL, but *
- * just a clarification of how we interpret "derived works" as it applies  *
- * to our GPL-licensed Nmap product.  This is similar to the way Linus     *
- * Torvalds has announced his interpretation of how "derived works"        *
- * applies to Linux kernel modules.  Our interpretation refers only to     *
- * Nmap - we don't speak for any other GPL products.                       *
+ * works of Nmap, as well as other software we distribute under this       *
+ * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+ * but is meant to clarify our interpretation of derived works with some   *
+ * common examples.  Our interpretation applies only to Nmap--we don't     *
+ * speak for other people's GPL works.                                     *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included COPYING.OpenSSL file, and distribute linked      *
- * combinations including the two. You must obey the GNU GPL in all        *
+ * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+ * linked combinations including the two. You must obey the GNU GPL in all *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
  * but you are not obligated to do so.                                     *
@@ -76,17 +69,18 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to fyodor@insecure.org for possible incorporation into the main         *
+ * to nmap-dev@insecure.org for possible incorporation into the main       *
  * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
- * to reuse, modify, and relicense the code.  Nmap will always be          *
- * available Open Source, but this is important because the inability to   *
- * relicense code has caused devastating problems for other Free Software  *
- * projects (such as KDE and NASM).  We also occasionally relicense the    *
- * code to third parties as discussed above.  If you wish to specify       *
- * special license conditions of your contributions, just say so when you  *
- * send them.                                                              *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -97,21 +91,22 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: services.cc 9965 2008-09-05 02:06:29Z fyodor $ */
-
-#include <list>
-#include <map>
+/* $Id: services.cc 29861 2012-09-25 05:24:16Z david $ */
 
+#include "nmap.h"
 #include "services.h"
 #include "NmapOps.h"
 #include "charpool.h"
 #include "nmap_error.h"
 #include "utils.h"
 
+#include <list>
+#include <map>
+
 /* This structure is the key for looking up services in the
    port/proto -> service map. */
 struct port_spec {
-  int portno; /* Network byte order */
+  int portno;
   std::string proto;
 
   /* Sort in the usual nmap-services order. */
@@ -138,12 +133,13 @@ bool service_node_ratio_compare(const service_node& a, const service_node& b) {
 }
 
 extern NmapOps o;
-static int numtcpports = 0;
-static int numudpports = 0;
+static int numtcpports;
+static int numudpports;
+static int numsctpports;
 static std::map<port_spec, service_node> service_table;
 static std::list<service_node> services_by_ratio;
-static int services_initialized = 0;
-static int ratio_format = 0; // 0 = /etc/services no-ratio format. 1 = new nmap format
+static int services_initialized;
+static int ratio_format; // 0 = /etc/services no-ratio format. 1 = new nmap format
 
 static int nmap_services_init() {
   if (services_initialized) return 0;
@@ -160,6 +156,13 @@ static int nmap_services_init() {
   int ratio_n, ratio_d;
   char ratio_str[32];
 
+  numtcpports = 0;
+  numudpports = 0;
+  numsctpports = 0;
+  service_table.clear();
+  services_by_ratio.clear();
+  ratio_format = 0;
+
   if (nmap_fetchfile(filename, sizeof(filename), "nmap-services") != 1) {
 #ifndef WIN32
     error("Unable to find nmap-services!  Resorting to /etc/services");
@@ -194,7 +197,7 @@ static int nmap_services_init() {
   while(fgets(line, sizeof(line), fp)) {
     lineno++;
     p = line;
-    while(*p && isspace((int) *p))
+    while(*p && isspace((int) (unsigned char) *p))
       p++;
     if (*p == '#')
       continue;
@@ -231,8 +234,6 @@ static int nmap_services_init() {
       continue;
     }
 
-    portno = htons(portno);
-
     port_spec ps;
     ps.portno = portno;
     ps.proto = proto;
@@ -242,7 +243,7 @@ static int nmap_services_init() {
     i = service_table.find(ps);
     if (i != service_table.end()) {
       if (o.debugging)
-        error("Port %d proto %s is duplicated in services file %s", ntohs(portno), proto, filename);
+        error("Port %d proto %s is duplicated in services file %s", portno, proto, filename);
       continue;
     }
 
@@ -250,6 +251,8 @@ static int nmap_services_init() {
       numtcpports++;
     } else if (strncasecmp(proto, "udp", 3) == 0) {
       numudpports++;
+    } else if (strncasecmp(proto, "sctp", 4) == 0) {
+      numsctpports++;
     } else if (strncasecmp(proto, "ddp", 3) == 0) {
       /* ddp is some apple thing...we don't "do" that */
     } else if (strncasecmp(proto, "divert", 6) == 0) {
@@ -283,6 +286,13 @@ static int nmap_services_init() {
   return 0;
 }
 
+void free_services() {
+  /* This doesn't free anything, because the service_table is allocated
+     statically. It just marks the table as needing to be reinitialized because
+     other things have been freed, for example the cp_strdup-allocated members
+     of service_node. */
+  services_initialized = 0;
+}
 
   
 /* Adds ports whose names match mask and one or more protocols
@@ -302,11 +312,15 @@ int addportsfromservmask(char *mask, u8 *porttbl, int range_type) {
     service_node& current = i->second;
     if (wildtest(mask, current.s_name)) {
       if ((range_type & SCAN_TCP_PORT) && strcmp(current.s_proto, "tcp") == 0) {
-        porttbl[ntohs(current.s_port)] |= SCAN_TCP_PORT;
+        porttbl[current.s_port] |= SCAN_TCP_PORT;
         t++;
       }
       if ((range_type & SCAN_UDP_PORT) && strcmp(current.s_proto, "udp") == 0) {
-        porttbl[ntohs(current.s_port)] |= SCAN_UDP_PORT;
+        porttbl[current.s_port] |= SCAN_UDP_PORT;
+        t++;
+      }
+      if ((range_type & SCAN_SCTP_PORT) && strcmp(current.s_proto, "sctp") == 0) {
+        porttbl[current.s_port] |= SCAN_SCTP_PORT;
         t++;
       }
     }
@@ -317,7 +331,6 @@ int addportsfromservmask(char *mask, u8 *porttbl, int range_type) {
 
 
 
-/* Port must be in network byte order. */
 struct servent *nmap_getservbyport(int port, const char *proto) {
   std::map<port_spec, service_node>::iterator i;
   port_spec ps;
@@ -355,11 +368,15 @@ static bool is_port_member(const struct scan_lists *ptsdata, const struct servic
 
   if (strcmp(serv->s_proto, "tcp") == 0) {
     for (i=0; i<ptsdata->tcp_count; i++)
-      if (ntohs(serv->s_port) == ptsdata->tcp_ports[i])
+      if (serv->s_port == ptsdata->tcp_ports[i])
         return true;
   } else if (strcmp(serv->s_proto, "udp") == 0) {
     for (i=0; i<ptsdata->udp_count; i++)
-      if (ntohs(serv->s_port) == ptsdata->udp_ports[i])
+      if (serv->s_port == ptsdata->udp_ports[i])
+        return true;
+  } else if (strcmp(serv->s_proto, "sctp") == 0) {
+    for (i=0; i<ptsdata->sctp_count; i++)
+      if (serv->s_port == ptsdata->sctp_ports[i])
         return true;
   }
 
@@ -377,10 +394,10 @@ static bool is_port_member(const struct scan_lists *ptsdata, const struct servic
 // and return the N highest ratio ports (where N==level).
 //
 // This function doesn't support IP protocol scan so only call this
-// function if o.TCPScan() || o.UDPScan()
+// function if o.TCPScan() || o.UDPScan() || o.SCTPScan()
 
 void gettoppts(double level, char *portlist, struct scan_lists * ports) {
-  int ti=0, ui=0;
+  int ti=0, ui=0, si=0;
   struct scan_lists ptsdata = { 0 };
   bool ptsdata_initialized = false;
   const struct service_node *current;
@@ -429,6 +446,8 @@ void gettoppts(double level, char *portlist, struct scan_lists * ports) {
           ports->tcp_count++;
         else if (o.UDPScan() && strcmp(current->s_proto, "udp") == 0)
           ports->udp_count++;
+        else if (o.SCTPScan() && strcmp(current->s_proto, "sctp") == 0)
+          ports->sctp_count++;
       } else {
         break;
       }
@@ -440,6 +459,9 @@ void gettoppts(double level, char *portlist, struct scan_lists * ports) {
     if (ports->udp_count)
       ports->udp_ports = (unsigned short *)safe_zalloc(ports->udp_count * sizeof(unsigned short));
 
+    if (ports->sctp_count)
+      ports->sctp_ports = (unsigned short *)safe_zalloc(ports->sctp_count * sizeof(unsigned short));
+
     ports->prots = NULL;
 
     for (i = services_by_ratio.begin(); i != services_by_ratio.end(); i++) {
@@ -448,9 +470,11 @@ void gettoppts(double level, char *portlist, struct scan_lists * ports) {
         continue;
       if (current->ratio >= level) {
         if (o.TCPScan() && strcmp(current->s_proto, "tcp") == 0)
-          ports->tcp_ports[ti++] = ntohs(current->s_port);
+          ports->tcp_ports[ti++] = current->s_port;
         else if (o.UDPScan() && strcmp(current->s_proto, "udp") == 0)
-          ports->udp_ports[ui++] = ntohs(current->s_port);
+          ports->udp_ports[ui++] = current->s_port;
+        else if (o.SCTPScan() && strcmp(current->s_proto, "sctp") == 0)
+          ports->sctp_ports[si++] = current->s_port;
       } else {
         break;
       }
@@ -467,6 +491,10 @@ void gettoppts(double level, char *portlist, struct scan_lists * ports) {
       ports->udp_count = MIN((int) level, numudpports);
       ports->udp_ports = (unsigned short *)safe_zalloc(ports->udp_count * sizeof(unsigned short));
     }
+    if (o.SCTPScan()) {
+      ports->sctp_count = MIN((int) level, numsctpports);
+      ports->sctp_ports = (unsigned short *)safe_zalloc(ports->sctp_count * sizeof(unsigned short));
+    }
 
     ports->prots = NULL;
 
@@ -475,13 +503,16 @@ void gettoppts(double level, char *portlist, struct scan_lists * ports) {
       if (ptsdata_initialized && !is_port_member(&ptsdata, current))
         continue;
       if (o.TCPScan() && strcmp(current->s_proto, "tcp") == 0 && ti < ports->tcp_count)
-        ports->tcp_ports[ti++] = ntohs(current->s_port);
+        ports->tcp_ports[ti++] = current->s_port;
       else if (o.UDPScan() && strcmp(current->s_proto, "udp") == 0 && ui < ports->udp_count)
-        ports->udp_ports[ui++] = ntohs(current->s_port);
+        ports->udp_ports[ui++] = current->s_port;
+      else if (o.SCTPScan() && strcmp(current->s_proto, "sctp") == 0 && si < ports->sctp_count)
+        ports->sctp_ports[si++] = current->s_port;
     }
 
     if (ti < ports->tcp_count) ports->tcp_count = ti;
     if (ui < ports->udp_count) ports->udp_count = ui;
+    if (si < ports->sctp_count) ports->sctp_count = si;
   } else
     fatal("Argument to gettoppts (%g) should be a positive ratio below 1 or an integer of 1 or higher", level);
 
@@ -496,9 +527,12 @@ void gettoppts(double level, char *portlist, struct scan_lists * ports) {
   if (ports->udp_count > 1)
     qsort(ports->udp_ports, ports->udp_count, sizeof(unsigned short), &port_compare);
 
+  if (ports->sctp_count > 1)
+    qsort(ports->sctp_ports, ports->sctp_count, sizeof(unsigned short), &port_compare);
+
   if (o.debugging && level < 1)
-    log_write(LOG_STDOUT, "PORTS: Using ports open on %g%% or more average hosts (TCP:%d, UDP:%d)\n", level*100, ports->tcp_count, ports->udp_count);
+    log_write(LOG_STDOUT, "PORTS: Using ports open on %g%% or more average hosts (TCP:%d, UDP:%d, SCTP:%d)\n", level*100, ports->tcp_count, ports->udp_count, ports->sctp_count);
   else if (o.debugging && level >= 1)
-    log_write(LOG_STDOUT, "PORTS: Using top %d ports found open (TCP:%d, UDP:%d)\n", (int) level, ports->tcp_count, ports->udp_count);
+    log_write(LOG_STDOUT, "PORTS: Using top %d ports found open (TCP:%d, UDP:%d, SCTP:%d)\n", (int) level, ports->tcp_count, ports->udp_count, ports->sctp_count);
 }
 
