@@ -7,7 +7,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2012 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -17,11 +17,12 @@
  * technology into proprietary software, we sell alternative licenses      *
  * (contact sales@insecure.com).  Dozens of software vendors already       *
  * license Nmap technology such as host discovery, port scanning, OS       *
- * detection, and version detection.                                       *
+ * detection, version detection, and the Nmap Scripting Engine.            *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
- * misunderstandings, we consider an application to constitute a           *
+ * misunderstandings, we interpret that term as broadly as copyright law   *
+ * allows.  For example, we consider an application to constitute a        *
  * "derivative work" for the purpose of this license if it does any of the *
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
@@ -29,40 +30,32 @@
  *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
- *   not derivative works.)                                                * 
+ *   not derivative works.)                                                *
  * o Integrates/includes/aggregates Nmap into a proprietary executable     *
  *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
- * works of Nmap.  This list is not exclusive, but is just meant to        *
- * clarify our interpretation of derived works with some common examples.  *
- * These restrictions only apply when you actually redistribute Nmap.  For *
- * example, nothing stops you from writing and selling a proprietary       *
- * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://nmap.org to download Nmap.                                       *
- *                                                                         *
- * We don't consider these to be added restrictions on top of the GPL, but *
- * just a clarification of how we interpret "derived works" as it applies  *
- * to our GPL-licensed Nmap product.  This is similar to the way Linus     *
- * Torvalds has announced his interpretation of how "derived works"        *
- * applies to Linux kernel modules.  Our interpretation refers only to     *
- * Nmap - we don't speak for any other GPL products.                       *
+ * works of Nmap, as well as other software we distribute under this       *
+ * license such as Zenmap, Ncat, and Nping.  This list is not exclusive,   *
+ * but is meant to clarify our interpretation of derived works with some   *
+ * common examples.  Our interpretation applies only to Nmap--we don't     *
+ * speak for other people's GPL works.                                     *
  *                                                                         *
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
  * applications and appliances.  These contracts have been sold to dozens  *
  * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates as well as helping to     *
- * fund the continued development of Nmap technology.  Please email        *
- * sales@insecure.com for further information.                             *
+ * as providing for priority support and updates.  They also fund the      *
+ * continued development of Nmap.  Please email sales@insecure.com for     *
+ * further information.                                                    *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included COPYING.OpenSSL file, and distribute linked      *
- * combinations including the two. You must obey the GNU GPL in all        *
+ * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
+ * linked combinations including the two. You must obey the GNU GPL in all *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
  * but you are not obligated to do so.                                     *
@@ -78,17 +71,18 @@
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
- * to fyodor@insecure.org for possible incorporation into the main         *
+ * to nmap-dev@insecure.org for possible incorporation into the main       *
  * distribution.  By sending these changes to Fyodor or one of the         *
- * Insecure.Org development mailing lists, it is assumed that you are      *
- * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
- * to reuse, modify, and relicense the code.  Nmap will always be          *
- * available Open Source, but this is important because the inability to   *
- * relicense code has caused devastating problems for other Free Software  *
- * projects (such as KDE and NASM).  We also occasionally relicense the    *
- * code to third parties as discussed above.  If you wish to specify       *
- * special license conditions of your contributions, just say so when you  *
- * send them.                                                              *
+ * Insecure.Org development mailing lists, or checking them into the Nmap  *
+ * source code repository, it is understood (unless you specify otherwise) *
+ * that you are offering the Nmap Project (Insecure.Com LLC) the           *
+ * unlimited, non-exclusive right to reuse, modify, and relicense the      *
+ * code.  Nmap will always be available Open Source, but this is important *
+ * because the inability to relicense code has caused devastating problems *
+ * for other Free Software projects (such as KDE and NASM).  We also       *
+ * occasionally relicense the code to third parties as discussed above.    *
+ * If you wish to specify special license conditions of your               *
+ * contributions, just say so when you send them.                          *
  *                                                                         *
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
@@ -99,11 +93,12 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: timing.cc 9525 2008-08-16 01:05:11Z david $ */
+/* $Id: timing.cc 28413 2012-04-07 08:07:38Z david $ */
 
 #include "timing.h"
 #include "NmapOps.h"
 #include "utils.h"
+#include "xml.h"
 
 extern NmapOps o;
 
@@ -156,24 +151,24 @@ void adjust_timeouts2(const struct timeval *sent,
     to->srtt = delta;
     to->rttvar = MAX(5000, MIN(to->srtt, 2000000));
     to->timeout = to->srtt + (to->rttvar << 2);
-  }
-  else {
+  } else {
+    long rttdelta;
+
     if (delta >= 8000000 || delta < 0) {
       if (o.verbose)
-	error("%s: packet supposedly had rtt of %lu microseconds.  Ignoring time.", __func__, delta);
+	error("%s: packet supposedly had rtt of %ld microseconds.  Ignoring time.", __func__, delta);
       return;
     }
-    delta -= to->srtt;
+    rttdelta = delta - to->srtt;
     /* sanity check 2*/
-    if (delta > 1500000 && delta > 3 * to->srtt + 2 * to->rttvar) {
-      /* WANKER ALERT! */
+    if (rttdelta > 1500000 && rttdelta > 3 * to->srtt + 2 * to->rttvar) {
       if (o.debugging) {
-	log_write(LOG_STDOUT, "Bogus delta: %ld (srtt %d) ... ignoring\n", delta, to->srtt);
+	log_write(LOG_STDOUT, "Bogus rttdelta: %ld (srtt %d) ... ignoring\n", rttdelta, to->srtt);
       }
       return;
     }
-    to->srtt += delta >> 3;
-    to->rttvar += (ABS(delta) - to->rttvar) >> 2;
+    to->srtt += rttdelta >> 3;
+    to->rttvar += (ABS(rttdelta) - to->rttvar) >> 2;
     to->timeout = to->srtt + (to->rttvar << 2);  
   }
   if (to->rttvar > 2300000) {
@@ -238,6 +233,93 @@ void enforce_scan_delay(struct timeval *tv) {
   return;    
 }
 
+
+/* Returns the scaling factor to use when incrementing the congestion
+   window. */
+double ultra_timing_vals::cc_scale(const struct scan_performance_vars *perf) {
+  double ratio;
+
+  assert(num_replies_received > 0);
+  ratio = (double) num_replies_expected / num_replies_received;
+
+  return MIN(ratio, perf->cc_scale_max);
+}
+
+/* Update congestion variables for the receipt of a reply. */
+void ultra_timing_vals::ack(const struct scan_performance_vars *perf, double scale) {
+  num_replies_received++;
+
+  if (cwnd < ssthresh) {
+    /* In slow start mode. "During slow start, a TCP increments cwnd by at most
+       SMSS bytes for each ACK received that acknowledges new data." */
+    cwnd += perf->slow_incr * cc_scale(perf) * scale;
+    if (cwnd > ssthresh)
+      cwnd = ssthresh;
+  } else {
+    /* Congestion avoidance mode. "During congestion avoidance, cwnd is
+       incremented by 1 full-sized segment per round-trip time (RTT). The
+       equation
+         cwnd += SMSS*SMSS/cwnd
+       provides an acceptable approximation to the underlying principle of
+       increasing cwnd by 1 full-sized segment per RTT." */
+    cwnd += perf->ca_incr / cwnd * cc_scale(perf) * scale;
+  }
+  if (cwnd > perf->max_cwnd)
+    cwnd = perf->max_cwnd;
+}
+
+/* Update congestion variables for a detected drop. */
+void ultra_timing_vals::drop(unsigned in_flight,
+  const struct scan_performance_vars *perf, const struct timeval *now) {
+  /* "When a TCP sender detects segment loss using the retransmission timer, the
+     value of ssthresh MUST be set to no more than the value
+       ssthresh = max (FlightSize / 2, 2*SMSS)
+     Furthermore, upon a timeout cwnd MUST be set to no more than the loss
+     window, LW, which equals 1 full-sized segment (regardless of the value of
+     IW)." */
+  cwnd = perf->low_cwnd;
+  ssthresh = (int) MAX(in_flight / perf->host_drop_ssthresh_divisor, 2);
+  last_drop = *now;
+}
+
+/* Update congestion variables for a detected drop, but less aggressively for
+   group congestion control. */
+void ultra_timing_vals::drop_group(unsigned in_flight,
+  const struct scan_performance_vars *perf, const struct timeval *now) {
+  cwnd = MAX(perf->low_cwnd, cwnd / perf->group_drop_cwnd_divisor);
+  ssthresh = (int) MAX(in_flight / perf->group_drop_ssthresh_divisor, 2);
+  last_drop = *now;
+}
+
+/* Do initialization after the global NmapOps table has been filled in. */
+void scan_performance_vars::init() {
+  /* TODO: I should revisit these values for tuning.  They should probably
+     at least be affected by -T. */
+  low_cwnd = o.min_parallelism ? o.min_parallelism : 1;
+  max_cwnd = MAX(low_cwnd, o.max_parallelism ? o.max_parallelism : 300);
+  group_initial_cwnd = box(low_cwnd, max_cwnd, 10);
+  host_initial_cwnd = group_initial_cwnd;
+  slow_incr = 1;
+  /* The congestion window grows faster with more aggressive timing. */
+  if (o.timing_level < 4)
+    ca_incr = 1;
+  else
+    ca_incr = 2;
+  cc_scale_max = 50;
+  initial_ssthresh = 75;
+  group_drop_cwnd_divisor = 2.0;
+  /* Change the amount that ssthresh drops based on the timing level. */
+  double ssthresh_divisor;
+  if (o.timing_level <= 3)
+    ssthresh_divisor = (3.0 / 2.0);
+  else if (o.timing_level <= 4)
+    ssthresh_divisor = (4.0 / 3.0);
+  else
+    ssthresh_divisor = (5.0 / 4.0);
+  group_drop_ssthresh_divisor = ssthresh_divisor;
+  host_drop_ssthresh_divisor = ssthresh_divisor;
+}
+
 /* current_rate_history defines how far back (in seconds) we look when
    calculating the current rate. */
 RateMeter::RateMeter(double current_rate_history) {
@@ -276,7 +358,6 @@ void RateMeter::stop(const struct timeval *now) {
    now. If now is NULL, get the current time with gettimeofday. */
 void RateMeter::update(double amount, const struct timeval *now) {
   struct timeval tv;
-  static int clockwarn = 0;
   double diff;
   double interval;
   double count;
@@ -301,23 +382,17 @@ void RateMeter::update(double amount, const struct timeval *now) {
   /* How long since the last update? */
   diff = TIMEVAL_SUBTRACT(*now, last_update_tv) / 1000000.0;
 
-  /* On my SMP Linux 2.6.20 system, I'm getting very occasional values
-     like "now=1214173867.8027; last_update_tv=1214173867.8065".
-     Unless I'm missing something, I think my gettimeofday may have
-     decreased by 38 microseconds.  Perhaps due to SMP and the old
-     kernel.  Anyway, I think I'll just treat decreases of up to 1ms
-     as zero -Fyodor 
-     Updated to allow up to 5ms due to crash:
-     RateMeter::update: negative time delta; now=1217210189.144224; last_update_tv=1217210189.148486 */
-  if (diff < 0 && diff > -0.005) {
-    if (!clockwarn) {
-      error("Warning: RateMeter::update: negative time delta; now=%lu.%lu; last_update_tv=%lu.%lu", (unsigned long) now->tv_sec, (unsigned long) now->tv_usec, (unsigned long) last_update_tv.tv_sec, (unsigned long) last_update_tv.tv_usec);
-      clockwarn = 1;
-    }
+  if (diff < -current_rate_history)
+    /* This happened farther in the past than we care about. */
+    return;
+
+  if (diff < 0.0) {
+    /* If the event happened in the past, just add it into the total and don't
+       change last_update_tv, as if it had happened at the same time as the most
+       recent event. */
+    now = &last_update_tv;
     diff = 0.0;
   }
-  if (diff < 0.0)
-    fatal("RateMeter::update: negative time delta; now=%lu.%lu; last_update_tv=%lu.%lu", (unsigned long) now->tv_sec, (unsigned long) now->tv_usec, (unsigned long) last_update_tv.tv_sec, (unsigned long) last_update_tv.tv_usec);
 
   /* Find out how far back in time to look. We want to look back
      current_rate_history seconds, or to when the last update occurred,
@@ -350,7 +425,13 @@ void RateMeter::update(double amount, const struct timeval *now) {
 }
 
 double RateMeter::getOverallRate(const struct timeval *now) const {
-  return total / elapsedTime(now);
+  double elapsed;
+
+  elapsed = elapsedTime(now);
+  if (elapsed <= 0.0)
+    return 0.0;
+  else
+    return total / elapsed;
 }
 
 /* Get the "current" rate (actually a moving average of the last
@@ -444,7 +525,7 @@ ScanProgressMeter::ScanProgressMeter(const char *stypestr) {
   gettimeofday(&begin, NULL);
   last_print_test = begin;
   memset(&last_print, 0, sizeof(last_print));
-  memset(&last_est, 0, sizeof(last_print));
+  memset(&last_est, 0, sizeof(last_est));
   beginOrEndTask(&begin, NULL, true);
 }
 
@@ -461,7 +542,7 @@ ScanProgressMeter::~ScanProgressMeter() {
    might as well check this before spending much time computing
    progress info.  now can be NULL if caller doesn't have the current
    time handy.  Just because this function returns true does not mean
-   that the next printStatsIfNeccessary will always print something.
+   that the next printStatsIfNecessary will always print something.
    It depends on whether time estimates have changed, which this func
    doesn't even know about. */
 bool ScanProgressMeter::mayBePrinted(const struct timeval *now) {
@@ -477,40 +558,51 @@ bool ScanProgressMeter::mayBePrinted(const struct timeval *now) {
 
   if (last_print.tv_sec == 0) {
     /* We've never printed before -- the rules are less stringent */
-    if (TIMEVAL_MSEC_SUBTRACT(*now, begin) > 30000)
+    if (difftime(now->tv_sec, begin.tv_sec) > 30)
       return true;
-    else return false;
+    else
+      return false;
   } 
 
-  if (TIMEVAL_MSEC_SUBTRACT(*now, last_print_test) < 3000) 
+  if (difftime(now->tv_sec, last_print_test.tv_sec) < 3)
     return false;  /* No point even checking too often */
 
   /* We'd never want to print more than once per 30 seconds */
-  if (TIMEVAL_MSEC_SUBTRACT(*now, last_print) < 30000)
+  if (difftime(now->tv_sec, last_print.tv_sec) < 30)
     return false;
 
   return true;
 }
 
+/* Return an estimate of the time remaining if a process was started at begin
+   and is perc_done of the way finished. Returns inf if perc_done == 0.0. */
+static double estimate_time_left(double perc_done,
+                                 const struct timeval *begin,
+                                 const struct timeval *now) {
+  double time_used_s;
+  double time_needed_s;
+
+  time_used_s = difftime(now->tv_sec, begin->tv_sec);
+  time_needed_s = time_used_s / perc_done;
+
+  return time_needed_s - time_used_s;
+}
+
 /* Prints an estimate of when this scan will complete.  It only does
    so if mayBePrinted() is true, and it seems reasonable to do so
    because the estimate has changed significantly.  Returns whether
    or not a line was printed.*/
-bool ScanProgressMeter::printStatsIfNeccessary(double perc_done, 
+bool ScanProgressMeter::printStatsIfNecessary(double perc_done, 
 					       const struct timeval *now) {
   struct timeval tvtmp;
-  long time_used_ms;
-  long time_needed_ms;
-  long time_left_ms;
-  long prev_est_time_left_ms; /* Time left as per prev. estimate */
-  long change_abs_ms; /* absolute value of change */
+  double time_left_s;
   bool printit = false;
 
   if (!now) {
     gettimeofday(&tvtmp, NULL);
     now = (const struct timeval *) &tvtmp;
   }
-  
+
   if (!mayBePrinted(now))
     return false;
 
@@ -521,45 +613,39 @@ bool ScanProgressMeter::printStatsIfNeccessary(double perc_done,
 
   assert(perc_done <= 1.0);
 
-  /* OK, now lets estimate the time to finish */
-  time_used_ms = TIMEVAL_MSEC_SUBTRACT(*now, begin);
-  time_needed_ms = (int) ((double) time_used_ms / perc_done);
-  time_left_ms = time_needed_ms - time_used_ms;
+  time_left_s = estimate_time_left(perc_done, &begin, now);
 
-  if (time_left_ms < 30000)
+  if (time_left_s < 30)
     return false; /* No point in updating when it is virtually finished. */
 
-  /* If we have not printed before, or if our previous ETC has elapsed, print
-     a new one */
-  if (last_print.tv_sec < 0)
+  if (last_est.tv_sec == 0) {
+    /* We don't have an estimate yet (probably means a low completion). */
     printit = true;
-  else {
-    /* If the estimate changed by more than X minutes, and if that
-       change represents at least X% of the time remaining, print
-       it.  */
-    prev_est_time_left_ms = TIMEVAL_MSEC_SUBTRACT(last_est, *now);
-    change_abs_ms = ABS(prev_est_time_left_ms - time_left_ms);
-    if (prev_est_time_left_ms <= 0)
-      printit = true;
-    else if (o.debugging || (change_abs_ms > 180000 && change_abs_ms > .05 * MAX(time_left_ms, prev_est_time_left_ms)))
+  } else if (TIMEVAL_AFTER(*now, last_est)) {
+    /* The last estimate we printed has passed. Print a new one. */
+    printit = true;
+  } else {
+    /* If the estimate changed by more than 3 minutes, and if that change
+       represents at least 5% of the total time, print it. */
+    double prev_est_total_time_s = difftime(last_est.tv_sec, begin.tv_sec);
+    double prev_est_time_left_s = difftime(last_est.tv_sec, last_print.tv_sec);
+    double change_abs_s = ABS(prev_est_time_left_s - time_left_s);
+    if (o.debugging || (change_abs_s > 15 && change_abs_s > .05 * prev_est_total_time_s))
       printit = true;
   }
 
   if (printit) {
      return printStats(perc_done, now);
   } 
+
   return false;
 }
 
-
 /* Prints an estimate of when this scan will complete.  */
 bool ScanProgressMeter::printStats(double perc_done, 
                                    const struct timeval *now) {
   struct timeval tvtmp;
-  long time_used_ms;
-  long time_needed_ms;
-  long time_left_ms;
-  long sec_left;
+  double time_left_s;
   time_t timet;
   struct tm *ltime;
 
@@ -567,38 +653,46 @@ bool ScanProgressMeter::printStats(double perc_done,
     gettimeofday(&tvtmp, NULL);
     now = (const struct timeval *) &tvtmp;
   }
-  
-  /* OK, now lets estimate the time to finish */
-  time_used_ms = TIMEVAL_MSEC_SUBTRACT(*now, begin);
-  time_needed_ms = (int) ((double) time_used_ms / perc_done);
-  time_left_ms = time_needed_ms - time_used_ms;
 
-    /* Here we go! */
-    last_print = *now;
-    TIMEVAL_MSEC_ADD(last_est, *now, time_left_ms);
-    timet = last_est.tv_sec;
-    ltime = localtime(&timet);
-    assert(ltime);
+  last_print = *now;
 
-    sec_left = time_left_ms / 1000;
-
-    // If we're less than 1% done we probably don't have enough
-    // data for decent timing estimates. Also with perc_done == 0
-    // these elements will be nonsensical.
-    if (perc_done < 0.01) {
-      log_write(LOG_STDOUT, "%s Timing: About %.2f%% done\n", 
-                scantypestr, perc_done * 100);
-      log_flush(LOG_STDOUT);
-    } else {
-      log_write(LOG_STDOUT, "%s Timing: About %.2f%% done; ETC: %02d:%02d (%li:%02li:%02li remaining)\n", 
-                scantypestr, perc_done * 100, ltime->tm_hour, ltime->tm_min, sec_left / 3600, 
-                (sec_left % 3600) / 60, sec_left % 60);
-      log_write(LOG_XML, "<taskprogress task=\"%s\" time=\"%lu\" percent=\"%.2f\" remaining=\"%li\" etc=\"%lu\" />\n",
-		scantypestr, (unsigned long) now->tv_sec,
-		perc_done * 100, sec_left, (unsigned long) last_est.tv_sec);
-      log_flush(LOG_STDOUT|LOG_XML);
-    }
+  // If we're less than 1% done we probably don't have enough
+  // data for decent timing estimates. Also with perc_done == 0
+  // these elements will be nonsensical.
+  if (perc_done < 0.01) {
+    log_write(LOG_STDOUT, "%s Timing: About %.2f%% done\n",
+        scantypestr, perc_done * 100);
+    log_flush(LOG_STDOUT);
     return true;
+  }
+
+  /* Add 0.5 to get the effect of rounding in integer calculations. */
+  time_left_s = estimate_time_left(perc_done, &begin, now) + 0.5;
+
+  last_est = *now;
+  last_est.tv_sec += (time_t)time_left_s;
+
+  /* Get the estimated time of day at completion */
+  timet = last_est.tv_sec;
+  ltime = localtime(&timet);
+  assert(ltime);
+
+  log_write(LOG_STDOUT, "%s Timing: About %.2f%% done; ETC: %02d:%02d (%.f:%02.f:%02.f remaining)\n",
+      scantypestr, perc_done * 100, ltime->tm_hour, ltime->tm_min,
+      floor(time_left_s / 60.0 / 60.0),
+      floor(fmod(time_left_s / 60.0, 60.0)),
+      floor(fmod(time_left_s, 60.0)));
+  xml_open_start_tag("taskprogress");
+  xml_attribute("task", "%s", scantypestr);
+  xml_attribute("time", "%lu", (unsigned long) now->tv_sec);
+  xml_attribute("percent", "%.2f", perc_done * 100);
+  xml_attribute("remaining", "%.f", time_left_s);
+  xml_attribute("etc", "%lu", (unsigned long) last_est.tv_sec);
+  xml_close_empty_tag();
+  xml_newline();
+  log_flush(LOG_STDOUT|LOG_XML);
+ 
+  return true;
 }
 
 /* Indicates that the task is beginning or ending, and that a message should
@@ -623,22 +717,28 @@ bool ScanProgressMeter::beginOrEndTask(const struct timeval *now, const char *ad
   tm = localtime(&tv_sec);
   if (beginning) {
     log_write(LOG_STDOUT, "Initiating %s at %02d:%02d", scantypestr, tm->tm_hour, tm->tm_min);
-    log_write(LOG_XML, "<taskbegin task=\"%s\" time=\"%lu\"", scantypestr, (unsigned long) now->tv_sec);
+    xml_open_start_tag("taskbegin");
+    xml_attribute("task", "%s", scantypestr);
+    xml_attribute("time", "%lu", (unsigned long) now->tv_sec);
     if (additional_info) {
       log_write(LOG_STDOUT, " (%s)", additional_info);
-      log_write(LOG_XML, " extrainfo=\"%s\"", additional_info);
+      xml_attribute("extrainfo", "%s", additional_info);
     }
     log_write(LOG_STDOUT, "\n");
-    log_write(LOG_XML, " />\n");
+    xml_close_empty_tag();
+    xml_newline();
   } else {
     log_write(LOG_STDOUT, "Completed %s at %02d:%02d, %.2fs elapsed", scantypestr, tm->tm_hour, tm->tm_min, TIMEVAL_MSEC_SUBTRACT(*now, begin) / 1000.0);
-    log_write(LOG_XML, "<taskend task=\"%s\" time=\"%lu\"", scantypestr, (unsigned long) now->tv_sec);
+    xml_open_start_tag("taskend");
+    xml_attribute("task", "%s", scantypestr);
+    xml_attribute("time", "%lu", (unsigned long) now->tv_sec);
     if (additional_info) {
       log_write(LOG_STDOUT, " (%s)", additional_info);
-      log_write(LOG_XML, " extrainfo=\"%s\"", additional_info);
+      xml_attribute("extrainfo", "%s", additional_info);
     }
     log_write(LOG_STDOUT, "\n");
-    log_write(LOG_XML, " />\n");
+    xml_close_empty_tag();
+    xml_newline();
   }
   log_flush(LOG_STDOUT|LOG_XML);
   return true;
